#include "usart_driver.h"
#include "usart_comm.h"
#include "CLI.h"
#include "EEP_adresses.h"
#include "devices.h"
#include "COMM485.h"
#include "EXT_I2C.h"

#include "dev_PowerDist.h"
#include "dev_PowerSource.h"
#include "dev_Signal.h"
#include "dev_Trackswitch.h"
#include "dev_light.h"

#include <avr\io.h>
#include <stdio.h>
/*****************************/
/* FOR AVRGCC 4.33 or higher */
/*****************************/


/*************************************************************************/
/*  Select Type (SET/GET/Write/Read/Clear)				                 */
/*************************************************************************/
void CLI_PacketHandler(unsigned char *TxBuffer)
{
 if (GlobalPIB.Dir==UCOMM_DIR_Read) 
 {
    CLI_ReadHandler(TxBuffer);
    return;
 }
 if (GlobalPIB.Dir==UCOMM_DIR_Write) 
 {
    CLI_WriteHandler(TxBuffer);
    return;
 }
 if (GlobalPIB.Dir==UCOMM_DIR_SET) 
 {
    CLI_SETHandler(TxBuffer);
    return;
 }
 if (GlobalPIB.Dir==UCOMM_DIR_GET) 
 {
    CLI_GETHandler(TxBuffer);
    return;
 }
 if (GlobalPIB.Dir==UCOMM_DIR_CLEAR) 
 {
    CLI_CLRHandler(TxBuffer);
    return;
 }
 if (GlobalPIB.Dir==UCOMM_DIR_SCAN) 
 {
    CLI_SCANHandler(TxBuffer);
    return;
 }
 if (GlobalPIB.Dir==UCOMM_DIR_SEARCH) 
 {
    CLI_SEARCHHandler(TxBuffer);
    return;
 }
 if (GlobalPIB.Dir==UCOMM_DIR_MultiWrite)
 {
    CLI_MultiWriteHandler(TxBuffer);
    return;
 }   
 if (GlobalPIB.Dir==UCOMM_DIR_MultiRead)
 {
    CLI_MultiReadHandler(TxBuffer);
    return;
 }  
  // Nothing usefull found ???
  UCOMM_SendErrorpacket(TxBuffer,UCOMM_Error_CMDVAL);   // send back Error
}


/*************************************************************************/
/*                  */
/*************************************************************************/
void CLI_ReadHandler(unsigned char *TxBuffer)
{
    unsigned char Error;
    unsigned char ret;
    
    Error=UCOMM_Error_None;
    switch(GlobalPIB.CMD)
    {
        default:
                    Error=UCOMM_Error_CMDVAL;
                    break;
                    
       	case CLI_PCMD_Trackswitch:
					ret=Trackswitch_READ(TxBuffer);
					if (ret > UCOMM_Error_None) Error=ret;   // > 0x80 ??
					break;
					
		case CLI_PCMD_Decoupler:
        case CLI_PCMD_Light:
                    Error=UCOMM_Error_CMDVAL;
                    break;
        case CLI_PCMD_PowerSource:
                     Error=PowerSource_READ(TxBuffer);
                    
                    break; 
        case CLI_PCMD_Signal: 
                    ret=SIGNAL_GET(TxBuffer);
                    if (ret > UCOMM_Error_None) Error=ret;
                    break;
        case CLI_PCMD_Posdetect: 
                    break;
		case CLI_PCMD_PowerDist:   
                    ret=PowerDist_READ(TxBuffer);
		            Error=ret;
                    if (ret==EXT_I2C_Error_NONE)  Error=UCOMM_Error_None;
					break;                     
    }
    
    if (Error!=UCOMM_Error_None)  UCOMM_SendErrorpacket(TxBuffer,Error);
}
/*************************************************************************/
/*                  													 */
/*************************************************************************/
void CLI_WriteHandler(unsigned char *TxBuffer)
{
    unsigned char Error;
    unsigned char ret;
    
    Error=UCOMM_Error_None;
	
	switch(GlobalPIB.CMD)
    {
        default:
                    Error=UCOMM_Error_CMDVAL;
                    break;
		
		case CLI_PCMD_Trackswitch:
		            ret=Trackswitch_WRITE(TxBuffer);
                    Error=ret;
                    if (ret==EXT_I2C_Error_NONE)  Error=UCOMM_Error_None;
					break;        
		case CLI_PCMD_Decoupler:
                    break;
        
        case CLI_PCMD_Light:
                    if (GlobalPIB.Param_EE < 0x0A)
                    {
                        ret=Light_SET(TxBuffer);
                    }
                    else
                    {
                        ret=Light_SETREG(TxBuffer);                    
                    }                
                    Error=ret;
                    if (ret==EXT_I2C_Error_NONE)  Error=UCOMM_Error_None;
                    break;
        case CLI_PCMD_PowerSource:
                    ret=PowerSource_WRITE(TxBuffer);
                    if (ret > UCOMM_Error_None) Error=ret;
                    break;
        case CLI_PCMD_Signal: 
                    ret=SIGNAL_SET(TxBuffer);
                    if (ret > UCOMM_Error_None) Error=ret;
                    break;
        case CLI_PCMD_Posdetect: 
		           break;   
        case CLI_PCMD_PowerDist:   
                    ret=PowerDist_WRITE(TxBuffer);
		            Error=ret;
                    if (ret==EXT_I2C_Error_NONE)  Error=UCOMM_Error_None;
					break;                     
   
    }
   
    if (Error!=UCOMM_Error_None)  UCOMM_SendErrorpacket(TxBuffer,Error);

}
/*************************************************************************/
/*                  													 */
/*************************************************************************/
void CLI_MultiWriteHandler(unsigned char *TxBuffer)
{
    unsigned char Error;
    unsigned char ret;
    unsigned char cnt_dev;
    
    Error=UCOMM_Error_None;
	

 // check if num of parameters valid
 if (!GlobalMultiPIB.Param_Entrys) 
 {
     Error=UCOMM_Error_CMDVAL;
     UCOMM_SendErrorpacket(TxBuffer,Error);
     return;
 }
 
 for (cnt_dev=1;cnt_dev<(GlobalMultiPIB.Param_Entrys+1);cnt_dev++)
 {
 
    // Get Multi parameters
    
    GlobalPIB.DeviceNum=GlobalMultiPIB.DeviceNum[cnt_dev];
    GlobalPIB.Param_EE=GlobalMultiPIB.Param_EE[cnt_dev];
    GlobalPIB.Param_FF=GlobalMultiPIB.Param_FF[cnt_dev];
 
	switch(GlobalPIB.CMD)
    {
        default:
                    Error=UCOMM_Error_CMDVAL;
                    break;
		
		case CLI_PCMD_Trackswitch:
		            ret=Trackswitch_WRITE(TxBuffer);
                    Error=ret;
                    if (ret==EXT_I2C_Error_NONE)  Error=UCOMM_Error_None;
					break;        
        case CLI_PCMD_Light:
 
                    if (GlobalPIB.Param_EE < 0x0A)
                    {
                        ret=Light_SET_Multi(TxBuffer);
                    }
                    else
                    {
                        ret=Light_SETREG(TxBuffer);                    
                    }                
                    Error=ret;
                    if (ret==EXT_I2C_Error_NONE)  Error=UCOMM_Error_None;
                    break;
        case CLI_PCMD_Signal: 
                    ret=SIGNAL_SET(TxBuffer);
                    if (ret > UCOMM_Error_None) Error=ret;
                    break;
    }
 
 
   GlobalMultiPIB.Error[cnt_dev]=Error;
 }
 UCOMM_SendAnswerpacket_Multi(TxBuffer);
}
/*************************************************************************/
/*                  													 */
/*************************************************************************/
void CLI_MultiReadHandler(unsigned char *TxBuffer)
{
    unsigned char Error;
    unsigned char ret;
    unsigned char cnt_dev;
    
    Error=UCOMM_Error_None;
	

 // check if num of parameters valid
 if (!GlobalMultiPIB.Param_Entrys) 
 {
     Error=UCOMM_Error_CMDVAL;
     UCOMM_SendErrorpacket(TxBuffer,Error);
     return;
 }
 
 for (cnt_dev=1;cnt_dev<(GlobalMultiPIB.Param_Entrys+1);cnt_dev++)
 {
 
    // Get Multi parameters
    
    GlobalPIB.DeviceNum=GlobalMultiPIB.DeviceNum[cnt_dev];
    GlobalPIB.Param_EE=GlobalMultiPIB.Param_EE[cnt_dev];
    GlobalPIB.Param_FF=GlobalMultiPIB.Param_FF[cnt_dev];
 
	 switch(GlobalPIB.CMD)
    {
        default:
                    Error=UCOMM_Error_CMDVAL;
                    break;
                    
       	case CLI_PCMD_Trackswitch:
					ret=Trackswitch_READ(TxBuffer);
					if (ret > UCOMM_Error_None) Error=ret;   // > 0x80 ??
					break;
					
        case CLI_PCMD_Light:
                    Error=UCOMM_Error_CMDVAL;
                    break;
        case CLI_PCMD_Signal: 
                    ret=SIGNAL_GET(TxBuffer);
                    if (ret > UCOMM_Error_None) Error=ret;
                    break;
    }
 
 
   GlobalMultiPIB.Error[cnt_dev]=Error;
 }

}
/*************************************************************************/
/*                  													 */
/*************************************************************************/
void CLI_SETHandler(unsigned char *TxBuffer)
{
    unsigned char Error;
    Error=UCOMM_Error_None;
    switch(GlobalPIB.CMD)
    {
        default:
                    Error=UCOMM_Error_CMDVAL;
                    break;
        case CLI_PCMD_Baudrate:
                    CLI_SetBaudrate(TxBuffer);
                    
                    break; 
        case CLI_PCMD_S_PowerSource: 
                    UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
                    break;
        case CLI_PCMD_S_Trackswitch: 
        case CLI_PCMD_S_Decoupler:
        case CLI_PCMD_S_Light:
        case CLI_PCMD_S_Signal: 
        case CLI_PCMD_S_Posdetect: 
                    GlobalDIB.DeviceNum=GlobalPIB.DeviceNum;
                    GlobalDIB.RS485adress=GlobalPIB.Param_EES;
                    GlobalDIB.SUBadress=GlobalPIB.Param_FF;
                    GlobalDIB.SUBportbit=GlobalPIB.Param_GG;
                    DEV_4BYTE_WriteDIB(GlobalDIB.DeviceNum);
                    if (GlobalDIB.EEPAdress!=0)
                    {
                        UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
                    }
                    else
                    {
                        Error=UCOMM_Error_DevNumInvalid;       // Send Error
                    }
                    break;
		case CLI_PCMD_S_PowerDist:               
                    Error=PowerDist_SET(TxBuffer);
                    break;
        case CLI_PCMD_S_Loco:                    
                    GlobalDIB.DeviceNum=GlobalPIB.DeviceNum;
                    GlobalDIB.LocoAdress=GlobalPIB.Param_EES;
                    GlobalDIB.LocoConfig=GlobalPIB.Param_FF;
                    CLI_CopyPIBstr2DIBstr();              		// copy DIB2PIB String
					DEV_64BYTE_WriteDIB(GlobalDIB.DeviceNum);
                    if (GlobalDIB.EEPAdress!=0)
                    {
                        UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
                    }
                    else
                    {
                        Error=UCOMM_Error_DevNumInvalid;       // Send Error
                    }
                    break;
        case CLI_PCMD_S_NewI2CAdress:
                    Error=CLI_SetNewI2CAdresson485(TxBuffer);       
                    break;   
        case CLI_PCMD_S_DirectData:
                    Error=CLI_DirectData(TxBuffer);
                    break;
        }    
    if (Error!=UCOMM_Error_None)  UCOMM_SendErrorpacket_Setup(TxBuffer,Error);
    
}

/*************************************************************************/
/*                  */
/*************************************************************************/
void CLI_CLRHandler(unsigned char *TxBuffer)
{
    unsigned char Error;
    Error=UCOMM_Error_None;
    switch(GlobalPIB.CMD)
    {
        default:
                    Error=UCOMM_Error_CMDVAL;
                    break;
        case CLI_PCMD_Baudrate:
                    Error=UCOMM_Error_CMDVAL;
                    break; 
        
        case CLI_PCMD_S_Decoupler:
        case CLI_PCMD_S_Light:
                     Error=UCOMM_Error_DevNumInvalid;       // Send Error
                     break;
        case CLI_PCMD_S_PowerSource: 
                     UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
                     break;
        case CLI_PCMD_S_Signal: 
        case CLI_PCMD_S_Trackswitch: 
        case CLI_PCMD_S_Posdetect:
                    GlobalDIB.DeviceNum=GlobalPIB.DeviceNum;
                    GlobalDIB.RS485adress=0xFFFF;               // special "Clear" Adress
                    GlobalDIB.SUBadress=GlobalPIB.Param_FF;
                    GlobalDIB.SUBportbit=GlobalPIB.Param_GG;
                    DEV_4BYTE_WriteDIB(GlobalDIB.DeviceNum);
                    if (GlobalDIB.EEPAdress!=0)
                    {
                        UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
                    }
                    else
                    {
                        Error=UCOMM_Error_DevNumInvalid;       // Send Error
                    }
                    break;
		case CLI_PCMD_S_PowerDist:                
                    GlobalDIB.DeviceNum=GlobalPIB.DeviceNum;
                    GlobalDIB.RS485adress=0xFFFF;               // special "Clear" Adress
                    GlobalPIB.Param_GG=0xFF;                    // special "Clear" value
                    GlobalDIB.SUBadress=GlobalPIB.Param_FF;
                    GlobalDIB.SUBportbit=GlobalPIB.Param_GG;
                    DEV_4BYTE_WriteDIB(GlobalDIB.DeviceNum);
                    if (GlobalDIB.EEPAdress!=0)
                    {
                        UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
                    }
                    else
                    {
                        Error=UCOMM_Error_DevNumInvalid;       // Send Error
                    }
                    break;
         case CLI_PCMD_S_Loco:                    
                    GlobalDIB.DeviceNum=GlobalPIB.DeviceNum;
                    GlobalDIB.LocoAdress=0xFFFF;                // special "Clear" Adress
                    DEV_64BYTE_WriteDIB(GlobalDIB.DeviceNum);
                    if (GlobalDIB.EEPAdress!=0)
                    {
                        UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
                    }
                    else
                    {
                        Error=UCOMM_Error_DevNumInvalid;       // Send Error
                    }
                    break;

            
    
    }
    
    if (Error!=UCOMM_Error_None)  UCOMM_SendErrorpacket_Setup(TxBuffer,Error);
    
}

/*************************************************************************/
/*                  */
/*************************************************************************/
void CLI_SCANHandler(unsigned char *TxBuffer)
{
	unsigned int Scancnt;
	unsigned int FOUND;
    unsigned char Error;
    unsigned char retv;
    Error=UCOMM_Error_None;
    switch(GlobalPIB.CMD)
    {
        default:
                    Error=UCOMM_Error_CMDVAL;
                    break;
        
		/* SCAN for RS485 Device */
		case CLI_PCMD_SCAN0:
		       /* Controller itself */
               if (GlobalPIB.DeviceNum==0)  
               {
                 FOUND=CLI_BUS485_Specialadresses(); 
                 if ((GlobalPIB.Param_FF==0) && (GlobalPIB.Param_GG==0)) FOUND=0xFFFF;
			   }
				  
               /* Scan on BUS */
               if ((GlobalPIB.DeviceNum > 0) && (GlobalPIB.DeviceNum < 0xF00)) 
			   {
                    FOUND=0x00;                 		
                    for (Scancnt=GlobalPIB.DeviceNum;Scancnt<0xFFF;Scancnt++)
                    {
                       retv=BUS485_PING(Scancnt);   // Ping device on BUS     
                       if (retv != BUS485_Error_TimeoutSync)
                       {
                            FOUND=Scancnt;          // No timeout -> device avail
                            Scancnt=0xFFF0;         // exit Loop/For
                       }
                    }
                    if (!FOUND) GlobalPIB.DeviceNum=0xF00;        // Nothing found      
               }
              
               /* special on I2C */
               if (GlobalPIB.DeviceNum > 0xEFF)
               {
                    for (Scancnt=GlobalPIB.DeviceNum;Scancnt<0xFFF;Scancnt++)
                    {
                       GlobalPIB.DeviceNum=Scancnt;
                       FOUND=CLI_BUS485_Specialadresses(); 
                       if (FOUND) Scancnt=0xFFF0;   // exit Loop/For
                    }
                    if (!FOUND) FOUND=0xFFFF;       // Nothing found 
               }               
               
               break;
		
		
		/* SCAN for RS485 SUB-Device */
		case CLI_PCMD_SCAN1:
	
					break;
		
        
        /* Get Info on RS485 Device */
		case CLI_PCMD_SCAN2:
	          if ((GlobalPIB.Param_EES==0) || (GlobalPIB.Param_EES > 0xEFF)) 
		      {
                 CLI_BUS485_Specialadresses();
                 if ((GlobalPIB.Param_EE==0) && (GlobalPIB.Param_FF==0)) FOUND=0xFFFF;
			  }
			  else
			  {
                 CLI_GET485_DeviceInfo();
              }
			  break;

    }
    
    /* ANSWER or ERROR packet */
	if (Error!=UCOMM_Error_None)  
	{
		UCOMM_SendErrorpacket_Setup(TxBuffer,Error);
	}
	else
	{
	  GlobalPIB.DeviceNum=FOUND;				// set value		
	  GlobalPIB.Param_EES=FOUND;				// set value	
	  UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
  	}
    
}
/*************************************************************************/
/*                  */
/*************************************************************************/
void CLI_SEARCHHandler(unsigned char *TxBuffer)
{
   
	unsigned int Scancnt;
	unsigned int FOUND;
	unsigned char t8;
	
	unsigned char Error;
    Error=UCOMM_Error_None;
    
    FOUND=0xFFF;
    switch(GlobalPIB.CMD)
    {
        default:
                    Error=UCOMM_Error_CMDVAL;
                    break;
        
		case CLI_PCMD_S_Trackswitch: 
		      Scancnt=GlobalPIB.DeviceNum;
					FOUND=0xFFF;
					while(Scancnt<EEP_MaxNum_Trackswitch)
					{
						DEV_4BYTE_GetDIB(Scancnt);
						if (GlobalDIB.RS485adress < 0xFFE) 
						{
							FOUND=Scancnt;                      // store num
							Scancnt=EEP_MaxNum_Trackswitch;		// exit
						}		
						Scancnt++;
					}
		      break; 
        case CLI_PCMD_S_Decoupler:
                    Scancnt=GlobalPIB.DeviceNum;
					FOUND=0xFFF;
					while(Scancnt<EEP_MaxNum_Decoupler)
					{
						DEV_4BYTE_GetDIB(Scancnt);
						if (GlobalDIB.RS485adress < 0xFFE) 
						{
							FOUND=Scancnt;                      // store num
							Scancnt=EEP_MaxNum_Decoupler;		// exit
						}		
						Scancnt++;
					}
		            break; 
        case CLI_PCMD_S_Light:
                    Scancnt=GlobalPIB.DeviceNum;
					FOUND=0xFFF;
					while(Scancnt<EEP_MaxNum_Light)
					{
						DEV_4BYTE_GetDIB(Scancnt);
						if (GlobalDIB.RS485adress < 0xFFE) 
						{
							FOUND=Scancnt;                        // store num
							Scancnt=EEP_MaxNum_Light;			  // exit
						}		
						Scancnt++;
					}
		            break;  
		case CLI_PCMD_S_PowerDist:
		            Scancnt=GlobalPIB.DeviceNum;
					FOUND=0xFFF;
					while(Scancnt<EEP_MaxNum_PowerDist)
					{
						DEV_4BYTE_GetDIB(Scancnt);
                        if (!PowerDist_Test_DIB_invalid())       // valid
						{
							FOUND=Scancnt;                       // store num
							Scancnt=EEP_MaxNum_PowerDist;		 // exit
						}		
						Scancnt++;
					}
		            break;
      case CLI_PCMD_S_PowerSource: 
                    Scancnt=GlobalPIB.DeviceNum;
					FOUND=0xFFF;
					while(Scancnt<EEP_MaxNum_PowerSupply+1)
					{
						if (SYS_Detected_PowerSourceAn[Scancnt]) 
						{
							FOUND=Scancnt;                      // store num
							Scancnt=EEP_MaxNum_PowerSupply;		 // exit
						}		
						Scancnt++;
					}
		  break;  
      case CLI_PCMD_S_Signal: 
					Scancnt=GlobalPIB.DeviceNum;
					FOUND=0xFFF;
					while(Scancnt<EEP_MaxNum_Signals)
					{
						DEV_4BYTE_GetDIB(Scancnt);
						if (GlobalDIB.RS485adress < 0xFFE) 
						{
							FOUND=Scancnt;                      // store num
							Scancnt=EEP_MaxNum_Signals;			// exit
						}		
						Scancnt++;
					}
		            break;  
        case CLI_PCMD_S_Posdetect:  
					Scancnt=GlobalPIB.DeviceNum;
					FOUND=0xFFF;
					while(Scancnt<EEP_MaxNum_Posdetect)
					{
						DEV_4BYTE_GetDIB(Scancnt);
						if (GlobalDIB.RS485adress < 0xFFE) 
						{
							FOUND=Scancnt;                      // store num
							Scancnt=EEP_MaxNum_Posdetect;		// exit
						}		
						Scancnt++;
					}
		            break;  
        case CLI_PCMD_S_Loco:
					Scancnt=GlobalPIB.DeviceNum;
					FOUND=0xFFF;
					while(Scancnt<EEP_MaxNum_Loco)
					{
						DEV_64BYTE_GetDIB_short(Scancnt);
						if (GlobalDIB.LocoAdress < 0xFFE) 
						{
							FOUND=Scancnt;              // store num
							Scancnt=EEP_MaxNum_Loco;	// exit
						}		
						Scancnt++;
					}
		            break;  		
    }
    
    
    if (Error!=UCOMM_Error_None)  
	{
		UCOMM_SendErrorpacket_Setup(TxBuffer,Error);
	}
	else
	{
	  GlobalPIB.DeviceNum=FOUND;				// set value		
	  UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
  	}
}
/*************************************************************************/
/*                  */
/*************************************************************************/
void CLI_GETHandler(unsigned char *TxBuffer)
{
    unsigned char t8;
    unsigned char Error;
	Error=UCOMM_Error_None;
	
    
    switch(GlobalPIB.CMD)
    {
        default:
                    Error=UCOMM_Error_CMDVAL;
                    break;
        case CLI_PCMD_S_PowerSource:
                     Error=PowerSource_GET(TxBuffer);
                    break;
                            
        case CLI_PCMD_S_Trackswitch: 
        case CLI_PCMD_S_Decoupler:
        case CLI_PCMD_S_Light:
        case CLI_PCMD_S_Signal: 
        case CLI_PCMD_S_Posdetect: 
                    DEV_4BYTE_GetDIB(GlobalPIB.DeviceNum);
                    GlobalPIB.Param_EES=GlobalDIB.RS485adress;
                    GlobalPIB.Param_FF=GlobalDIB.SUBadress;
                    GlobalPIB.Param_GG=GlobalDIB.SUBportbit;
                    
                    if ((GlobalDIB.EEPAdress!=0) && (GlobalDIB.RS485adress<0xFF0)) 
                    {
                        UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
                    }
                    else
                    {
                        Error=UCOMM_Error_DevNumInvalid;       // Send Error
                    }
                    break;
		case CLI_PCMD_S_PowerDist:         
                    Error=PowerDist_GET(TxBuffer);
                    break;
        case CLI_PCMD_S_Loco:
                    DEV_64BYTE_GetDIB_long(GlobalPIB.DeviceNum);
                    CLI_CopyDIBstr2PIBstr();           		// copy DIB2PIB String
					if (GlobalDIB.EEPAdress!=0)
                    {
                        UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
                    }
                    else
                    {
                        Error=UCOMM_Error_DevNumInvalid;       // Send Error
                    }
                    break;
    
    }
    
    if (Error!=UCOMM_Error_None)  UCOMM_SendErrorpacket_Setup(TxBuffer,Error);
}

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
unsigned char CLI_SetNewI2CAdresson485(unsigned char *TxBuffer)
{
    
    unsigned char Err;
    unsigned char ret;
    unsigned char Buffer[16];
    
    // GlobalPIB.Param_EES;            // RS485 Adress
    // GlobalPIB.Param_FF;             // "old" I2C-Adress
    // GlobalPIB.Param_GG;             // "new" I2C-Adress
    
    DEV_4BYTE_GetDIB(GlobalPIB.DeviceNum);
    if (GlobalPIB.Param_EES==0xFFF) 
    {
         GlobalPIB.Param_EES=GlobalDIB.RS485adress;
    }
    if (GlobalPIB.Param_FF==0xFF)
    {
         GlobalPIB.Param_FF=GlobalDIB.SUBadress;
    }

    Err=UCOMM_Error_None;
    
	ret=BUS485_WriteData_Packet(GlobalPIB.Param_EES,&Buffer[0],0,CMD_RS485_SET_NEW_ADRESS_I2C,GlobalPIB.Param_FF,GlobalPIB.Param_GG);
	// BUS485_Error_TimeoutSync, Error or data(0) 
	
    if (ret==BUS485_Error_TimeoutSync)              Err=ret;
	if ((ret==0x10) || (ret > UCOMM_Error_None))    Err=ret;
	
    GlobalPIB.Param_EE=ret;

	if (Err!=UCOMM_Error_None) 
	{
        return(Err);
	}
	else
	{		
		UCOMM_SendAnswerpacket(TxBuffer);
        return(UCOMM_Error_None);			
	}	

}
/*************************************************************************/
/*                                                                       */
/*************************************************************************/
unsigned char CLI_DirectData(unsigned char *TxBuffer)
{
    
    unsigned char Err;
    unsigned char ret;
    unsigned char cnt;
    unsigned char CHK;
    unsigned char Buffer[96];
    unsigned int  COMMAND;
    
    // GlobalPIB.Param_EES;            // RS485 Adress or 0xFF
    // GlobalPIB.Param_FF;             // Length of additional data
    // GlobalPIB.Param_GG;             // Checksum of additional data
    // StringParam[0]                  // Command MSB
    // StringParam[1]                  // Command LSB
    // StringParam[2]                  // PARAM0
    // StringParam[3]                  // PARAM1
    // StringParam[4]                  // DATA 0
    
    DEV_4BYTE_GetDIB(GlobalPIB.DeviceNum);
    if (GlobalPIB.Param_EES==0xFFF) 
    {
         GlobalPIB.Param_EES=GlobalDIB.RS485adress;
    }
    
    /* CHECKSUM */
    CHK=0;
    for (cnt=0;cnt < GlobalPIB.Param_FF;cnt++)
    {
      CHK+=GlobalPIB.StringParam[cnt];                  // calc CHECKs
      Buffer[cnt]=GlobalPIB.StringParam[cnt+4];         // copy data
    }
    if (CHK != GlobalPIB.Param_GG)      return(UCOMM_Error_Checksum);
    
 
    /* COMMAND */
    COMMAND=GlobalPIB.StringParam[0];
    COMMAND=COMMAND << 8;
    COMMAND+=GlobalPIB.StringParam[1];
 
    Err=UCOMM_Error_None;
    
	ret=BUS485_WriteData_Packet(GlobalPIB.Param_EES,&Buffer[0],GlobalPIB.Param_FF-4,COMMAND,GlobalPIB.StringParam[2],GlobalPIB.StringParam[3]);
	// BUS485_Error_TimeoutSync, Error or data(0) 
	
    if (ret==BUS485_Error_TimeoutSync)              Err=ret;
	if ((ret==0x10) || (ret > UCOMM_Error_None))    Err=ret;
	
    GlobalPIB.Param_EE=ret;

	if (Err!=UCOMM_Error_None) 
	{
        return(Err);
	}
	else
	{		
		UCOMM_SendAnswerpacket(TxBuffer);
        return(UCOMM_Error_None);			
	}	

}

/**----------------------------------------------------------------------------**/
/** Command functions                                                          **/
/**----------------------------------------------------------------------------**/
/*************************************************************************/
/*                  */
/*************************************************************************/
void CLI_SetBaudrate(unsigned char *TxBuffer)
{
  if ((GlobalPIB.Param_FF==CLI_Param_BaudrateM_115k) && (GlobalPIB.Param_GG==CLI_Param_BaudrateL_115k))
  {
   USART_RS232_Disable();
   USART_Baudrate_Set(&USART_RS232,0,UART_BAUD_115200_BSEL,UART_BAUD_115200_Double);
   USART_RS232_Enable();
   UCOMM_SendAnswerpacket_Setup(TxBuffer); 
  
   return;
  }  
  if ((GlobalPIB.Param_FF==CLI_Param_BaudrateM_19k2) && (GlobalPIB.Param_GG==CLI_Param_BaudrateL_19k2))
  {
   USART_Baudrate_Set(&USART_RS232,0,UART_BAUD_19200_BSEL,UART_BAUD_19200_Double);
   UCOMM_SendAnswerpacket_Setup(TxBuffer);
   return;
  }  
  
  // not returned -> baudrate parameter unknown !
  UCOMM_SendErrorpacket_Setup(TxBuffer,UCOMM_Error_CMDVAL);

}
/*************************************************************************/
/*                  */
/*************************************************************************/
void CLI_CopyDIBstr2PIBstr(void)
{
      unsigned char cnt;
      cnt=0;
      while(cnt<64)
      {
	  	  if (GlobalDIB.STRING[cnt]=='%')  GlobalDIB.STRING[cnt]=0x00;
	  	  GlobalPIB.StringParam[cnt]=GlobalDIB.STRING[cnt];
		  if (GlobalDIB.STRING[cnt]==0x00)			cnt=64;
		  cnt++;
      }
}
void CLI_CopyPIBstr2DIBstr(void)
{
      unsigned char cnt;
      cnt=0;
      while(cnt<64)
      {
	  	  if (GlobalPIB.StringParam[cnt]=='%')  GlobalPIB.StringParam[cnt]=0x00;
	  	  GlobalDIB.STRING[cnt]=GlobalPIB.StringParam[cnt];
		  if (GlobalPIB.StringParam[cnt]==0x00)			cnt=64;
		  
		  cnt++;
      }

}


/**----------------------------------------------------------------------------**/
/** Device functions                                                           **/
/**----------------------------------------------------------------------------**/
/*************************************************************************/
/*                  */
/*************************************************************************/
unsigned int CLI_BUS485_Specialadresses(void)
{
	// use GloablPIB for data I/O
	
	
	/* Controller itself */
	if (GlobalPIB.DeviceNum==0)
	{
		GlobalPIB.Param_EE=0x55;
		GlobalPIB.Param_FF=0xAA;
		sprintf(&GlobalPIB.StringParam[0],"Master Controller");
		return(0);
	}

    /* OpenDCC Controller  */
    if ((GlobalPIB.DeviceNum>0xFEF) && (GlobalPIB.DeviceNum<0xFF8))
	{
		if (IsBitSETinVar_uchar(SYS_Detected_OpenDCC,GlobalPIB.DeviceNum-0xFF0))
		{
			GlobalPIB.Param_EE=0xFF;
			GlobalPIB.Param_FF=0x01;
			sprintf(&GlobalPIB.StringParam[0],"OpenDCC");
			return(GlobalPIB.DeviceNum);
		}	
	}

    /* Analog Power Source */
	if ((GlobalPIB.DeviceNum>0xEFF) && (GlobalPIB.DeviceNum<0xF08)) 
	{
		if (SYS_Detected_PowerSourceAn[GlobalPIB.DeviceNum-0xF00])
		{
			GlobalPIB.Param_EE=0x01;
			GlobalPIB.Param_FF=0xA1;
			sprintf(&GlobalPIB.StringParam[0],"Analog PowerSource");
			return(GlobalPIB.DeviceNum);
		}
	}


	/* PCA9555 for PowerDist */
	if ((GlobalPIB.DeviceNum>0xF1F) && (GlobalPIB.DeviceNum<0xF28))
	{
		if (SYS_Detected_PowerDist[GlobalPIB.DeviceNum-0xF20])
		{
			GlobalPIB.Param_EE=0x01;
			GlobalPIB.Param_FF=0xA2;
			sprintf(&GlobalPIB.StringParam[0],"Power Distributor");
			return(GlobalPIB.DeviceNum);
		}	
	}

  //GlobalPIB.DeviceNum=0xFFF;
  GlobalPIB.Param_FF=0x00;
  GlobalPIB.Param_EE=0x00;
  return(0);

}

/*************************************************************************/
/*                  */
/*************************************************************************/
void CLI_GET485_DeviceInfo(void)
{
  //unsigned char BUS485_WriteData_Packet(unsigned int Adress, unsigned char *TxBuf,unsigned int Txbuflen,unsigned int CMD, unsigned char Param1, unsigned char Param2)
  //unsigned char BUS485_ReadData_Packet(unsigned int Adress, unsigned char *RxBuf,unsigned int CMD, unsigned char Param1, unsigned char Param2)
  
  unsigned char tbuf[70];
  unsigned char cnt;
  BUS485_ReadData_Packet(GlobalPIB.Param_EES,&tbuf[0],COMM485_CMD_GETINFO,0,0);
  GlobalPIB.Param_EE=tbuf[0];
  GlobalPIB.Param_FF=tbuf[1];
  for (cnt=0;cnt<64;cnt++)
  {
     GlobalPIB.StringParam[cnt]=tbuf[cnt+2]; 
     if (!tbuf[cnt+2])  break;                    // exit loop, if 0 detected
  }
  //return(GlobalPIB.DeviceNum);
  return;
}
/*************************************************************************/
/*                  */
/*************************************************************************/
unsigned char IsBitSETinVar_uchar(unsigned char Var, unsigned char Bitnum)
{
	switch(Bitnum)
	{
		default:	break;
		case 0:		if (Var & (1<<0)) return(1);break;
		case 1:		if (Var & (1<<1)) return(1);break;
		case 2:		if (Var & (1<<2)) return(1);break;
		case 3:		if (Var & (1<<3)) return(1);break;
		case 4:		if (Var & (1<<4)) return(1);break;
		case 5:		if (Var & (1<<5)) return(1);break;
		case 6:		if (Var & (1<<6)) return(1);break;
		case 7:		if (Var & (1<<7)) return(1);break;
	}
	return(0);
}

unsigned char SETBITinVar_uchar(unsigned char Var, unsigned char Bitnum)
{
	switch(Bitnum)
	{
		default:	break;
		case 0:		Var |= (1<<0); return(Var);break;
		case 1:		Var |= (1<<1); return(Var);break;
		case 2:		Var |= (1<<2); return(Var);break;
		case 3:		Var |= (1<<3); return(Var);break;
		case 4:		Var |= (1<<4); return(Var);break;
		case 5:		Var |= (1<<5); return(Var);break;
		case 6:		Var |= (1<<6); return(Var);break;
		case 7:		Var |= (1<<7); return(Var);break;
	}
	return(Var);
}
unsigned char CLRBITinVar_uchar(unsigned char Var, unsigned char Bitnum)
{
	switch(Bitnum)
	{
		default:	break;
		case 0:		Var &=~(1<<0); return(Var);break;
		case 1:		Var &=~(1<<1); return(Var);break;
		case 2:		Var &=~(1<<2); return(Var);break;
		case 3:		Var &=~(1<<3); return(Var);break;
		case 4:		Var &=~(1<<4); return(Var);break;
		case 5:		Var &=~(1<<5); return(Var);break;
		case 6:		Var &=~(1<<6); return(Var);break;
		case 7:		Var &=~(1<<7); return(Var);break;
	}
	return(Var);
}

