
#include "devices.h"
#include "DCC.h"
#include "CLI.h"
#include "usart_driver.h"

/*************************************************************************/
/*    */
/*************************************************************************/
unsigned char DCC_SetLocoSpeed(unsigned char LocoDEVNUM, unsigned char LocoSpeed, unsigned char DIR, unsigned char LIGHT)
{
       // LcoDEVNUM is the devicenumber of the Loco, NOT the Adress !
       // LocoSpeed is the Speed in % -> 0..127%
      
       unsigned int r;
       unsigned char tbuf[16];
       
       if (LocoSpeed>127) LocoSpeed=127;
       
       DEV_64BYTE_GetDIB_short(LocoDEVNUM);     // get Loco adress from EEPROM
       r=DCC_SearchConfiguredListEntry(LocoDEVNUM);
       
       // is not known....
       if (!r)
       {
          // Set Loco in openDCC
           tbuf[0]='X';
           tbuf[1]=0x86;
           tbuf[2]=GlobalDIB.LocoAdress >> 8;
           tbuf[3]=GlobalDIB.LocoAdress & 0xFF;
           tbuf[4]=2;                               // MODE = DCC
           tbuf[5]=DCC_Map_CONFIG2Speed(GlobalDIB.LocoConfig);

           r=DCC_SendData2ODCC(&tbuf[0],&tbuf[0],6,1,DCC_COMM_Timeout);
           if (r==DCC_Error_Timeout)    
           {
                  return(DCC_Error_TimeoutByte);
           }    
       
          // Store in List
          DCC_AddConfiguredListEntry(LocoDEVNUM);
       }
       
       //* Set LOCO speed now *//
       tbuf[0]='X';
       tbuf[1]=0x80;
       tbuf[2]=GlobalDIB.LocoAdress >> 8;
       tbuf[3]=GlobalDIB.LocoAdress & 0xFF;
       tbuf[4]=LocoSpeed;
       
       tbuf[5]=0;
       if (DIR)     tbuf[5]|=(1<<5);
       if (LIGHT)   tbuf[5]|=(1<<4);
      
       r=DCC_SendData2ODCC(&tbuf[0],&tbuf[0],6,1,DCC_COMM_Timeout);
       if (r==DCC_Error_Timeout)    
       {
            return(DCC_Error_TimeoutByte);
       }    
       
       return(DCC_Error_None);

}

/*************************************************************************/
/*    */
/*************************************************************************/
unsigned char DCC_SendData2ODCC(unsigned char *Txbuf, unsigned char *Rxbuf,unsigned char Txlen, unsigned char Rxlen,unsigned int Timeout)
{
    // timout in 50s increments
    
    unsigned char cnt;
    
    if (Txlen)
    {
        for (cnt=0;cnt < Txlen; cnt++ )
        {
            USART_ODCC_TxByte(*(Txbuf++));
        }
        USART_ODCC_WaitTxByteDone();
    }
    USART_ODCC_RxBufCntr=0;
    if (Rxlen)
    {
        USART_ODCC_Timeout=Timeout;
        while(USART_ODCC_RxBufCntr<Rxlen)
        {
            if (!USART_ODCC_Timeout) return(DCC_Error_Timeout);
        }
    }
    
    for (cnt=0;cnt<Rxlen;cnt++) *(Rxbuf++)=USART_ODCC_RxBuf[cnt];
    
    return(DCC_Error_NONE);
}


/*************************************************************************/
/*    */
/*************************************************************************/
unsigned char DCC_AddConfiguredListEntry(unsigned int Devnum)
{
    // 0 <- free place in list
    unsigned char cnt;
    for (cnt=1;cnt<32;cnt++)
    {
       if (!DCC_ConfiguredLokos[cnt]) 
       {
            DCC_ConfiguredLokos[cnt]=Devnum;
            return(cnt);
       }  
    }
    // no free place -> replace old one
    cnt=DCC_ConfiguredLokos[0]+1;   // get last replaced +1
    if (cnt > 31) cnt=0;            // max reached -> restart
    DCC_ConfiguredLokos[cnt]=Devnum;
    DCC_ConfiguredLokos[0]=cnt;     // store last replaced
    return(1);
}
/*************************************************************************/
/*    */
/*************************************************************************/
unsigned int DCC_SearchConfiguredListEntry(unsigned int Devnum)
{
    // 0 <- not in list
    unsigned char cnt;
    for (cnt=1;cnt<32;cnt++)
    {
       if (DCC_ConfiguredLokos[cnt]==Devnum)  return(cnt); 
    }
    return(0);  // return "not found in list"
}
/*************************************************************************/
/*    */
/*************************************************************************/

unsigned char DCC_Map_CONFIG2Speed(unsigned char Config)
{
    if (!(Config & DCC_SPST1) && (!(Config & DCC_SPST0))) return(DCC_LocoSpeedmode_14);
    if (!(Config & DCC_SPST1) && (Config & DCC_SPST0))    return(DCC_LocoSpeedmode_27);
    if ((Config & DCC_SPST1) && (!(Config & DCC_SPST0)))  return(DCC_LocoSpeedmode_28);
    if ((Config & DCC_SPST1) && ((Config & DCC_SPST0)))   return(DCC_LocoSpeedmode_127);
    return(0);      // input illegal !
}