
/**********************************************************************/
/* Polled and Pin toggeling TWI Drivere for max. 2 TWI's              */
/**********************************************************************/

#include "TWI_M_P.h"
#include "SysTimer.h"


/**********************************************************************/
/* Init a TWI                                                         */
/**********************************************************************/
 
void TWI_MasterInit(unsigned char InterFaceNum,unsigned char TWI_Baudrate)
{
  TWI_BaudrateSet[InterFaceNum]=TWI_Baudrate;
  if (InterFaceNum==0)
  {
    TWI_IF0_SDA_Init;
    TWI_IF0_SCL_Init;
    TWI_Master_STOP_IF0();
  }
  if (InterFaceNum==1)
  {
    TWI_IF1_SDA_Init;
    TWI_IF1_SCL_Init;
    TWI_Master_STOP_IF1();
  }

}
/**********************************************************************/
/* Write/Read n Byte(s)                                               */
/**********************************************************************/
unsigned char TWI_MasterWriteRead(unsigned char InterFaceNum,
                                  unsigned char Adress,
                                  unsigned char *BufferWR,
                                  unsigned char *BufferRD,
                                  unsigned char WRNum,
                                  unsigned char RDNum)
{
 unsigned char ACK;
 unsigned char cnt;
 /* Interface 0 */
 if (InterFaceNum==0)
 {
    if (WRNum)      // something to write ?
    {
      TWI_Master_START_IF0();
      ACK=TWI_Master_WriteByte_IF0((Adress << 1));                // RW\ = 0  
      if (ACK==TWIDRV_Status_NACK) 
      {
        TWI_Master_STOP_IF0();
        return(TWIDRV_Status_NACK);                               // End because of NACK
      }
      for (cnt=0;cnt<WRNum;cnt++)
      {
        TWIDelayInterByte_IF0();
        ACK=TWI_Master_WriteByte_IF0(*(BufferWR+cnt));
      }
      TWI_Master_STOP_IF0();  
    }                                                             // End Write
    
    
    
    if (RDNum)    // something to read ?
    {
      TWI_Master_START_IF0();
      ACK=TWI_Master_WriteByte_IF0((Adress << 1)+0x01);           // RW\ = 1
      if (ACK==TWIDRV_Status_NACK) 
      {
        TWI_Master_STOP_IF0();
        return(TWIDRV_Status_NACK);                               // End because of NACK
      }
      if (RDNum > 1)
      {
        for (cnt=0;cnt<(RDNum-1);cnt++)
        {
           TWIDelayInterByte_IF0();
           ACK=TWI_Master_ReadByte_IF0((BufferRD+cnt));
           TWI_Master_ReadByteACK_IF0();                          // ACK      
        }
      }
      TWIDelayInterByte_IF0();
      ACK=TWI_Master_ReadByte_IF0((BufferRD+(RDNum-1)));
      TWI_Master_ReadByteNACK_IF0();                              // NACK
      TWI_Master_STOP_IF0(); 
    }                                                             // End Read
      
 }
 /* Interface 1 */
 if (InterFaceNum==1)
 {
    if (WRNum)      // something to write ?
    {
      TWI_Master_START_IF1();
      ACK=TWI_Master_WriteByte_IF1((Adress << 1));                // RW\ = 0  
      if (ACK==TWIDRV_Status_NACK) 
      {
        TWI_Master_STOP_IF1();
        return(TWIDRV_Status_NACK);                               // End because of NACK
      }  
      for (cnt=0;cnt<WRNum;cnt++)
      {
        ACK=TWI_Master_WriteByte_IF1(*(BufferWR+cnt));
      }
      TWI_Master_STOP_IF1();  
    }                                                             // End Write
    if (RDNum)    // something to read ?
    {
      TWI_Master_START_IF1();
      ACK=TWI_Master_WriteByte_IF1((Adress << 1)+0x01);           // RW\ = 1
      if (ACK==TWIDRV_Status_NACK) 
      {
        TWI_Master_STOP_IF1();
        return(TWIDRV_Status_NACK);                               // End because of NACK
      }
      if (RDNum > 1)
      {
        for (cnt=0;cnt<(RDNum-1);cnt++)
        {
           ACK=TWI_Master_ReadByte_IF1((BufferRD+cnt));
           TWI_Master_ReadByteACK_IF1();                          // ACK      
        }
      }
      ACK=TWI_Master_ReadByte_IF1((BufferRD+(RDNum-1)));
      TWI_Master_ReadByteNACK_IF1();                              // NACK
      TWI_Master_STOP_IF1(); 
    }                                                             // End Read  
 }
 return(TWIDRV_Status_Done); 
}                                  
/****************************************************************************/
/*** Check Dev ACK (for EEP)                                              ***/
/****************************************************************************/
unsigned char TWI_Master_CheckDeviceACK(unsigned char InterFaceNum,unsigned char Adress)
{
 unsigned char ACK;
 if (InterFaceNum==0)
 {
      TWI_Master_START_IF0();
      ACK=TWI_Master_WriteByte_IF0((Adress << 1));                // RW\ = 0  
      TWI_Master_STOP_IF0();
      if (ACK==TWIDRV_Status_NACK) return(TWIDRV_Status_NACK);    // End because of NACK
      return(TWIDRV_Status_Ready);
 }
 if (InterFaceNum==1)
 {
      TWI_Master_START_IF1();
      ACK=TWI_Master_WriteByte_IF1((Adress << 1));                // RW\ = 0  
      TWI_Master_STOP_IF1();
      if (ACK==TWIDRV_Status_NACK) return(TWIDRV_Status_NACK);    // End because of NACK
      return(TWIDRV_Status_Ready);
 }
 return(TWIDRV_Status_NACK);
}
/****************************************************************************/
/*** Start Stop for IF0 & IF 1                                            ***/
/****************************************************************************/
void TWI_Master_START_IF0(void)
{
  TWI_IF0_SDA_LO;
  TWIDelayH_IF0();
  TWI_IF0_SCL_LO;
  TWIDelayH_IF0();
}
void TWI_Master_START_IF1(void)
{
  TWI_IF1_SDA_LO;
  TWIDelay_IF1();
  TWI_IF1_SCL_LO;
  TWIDelay_IF1();
}

void TWI_Master_STOP_IF0(void)
{
  TWIDelayH_IF0();
  TWI_IF0_SCL_HI;
  TWIDelayH_IF0();
  TWI_IF0_SDA_HI;
  TWIDelayH_IF0();
}
void TWI_Master_STOP_IF1(void)
{
  TWIDelay_IF1();
  TWI_IF1_SCL_HI;
  TWIDelay_IF1();
  TWI_IF1_SDA_HI;
  TWIDelay_IF1();
}

/****************************************************************************/
/*** Read for IF0 & IF 1                                                 ***/
/****************************************************************************/
unsigned char TWI_Master_ReadByte_IF0(unsigned char *data)
{
  unsigned char cnt;
  unsigned char di;
  di=0;
  
  TWI_IF0_SDA_HI;
  TWI_IF0_SCL_LO;
  for (cnt=0;cnt<8;cnt++)
    {
        di=di << 1;
        TWI___Delay_Bit_IF0;
        TWI_IF0_SCL_HI;
        TWI___Delay_H_IF0;
        #ifdef TWI_IF0_CLKStrectch
        if (!TWI_IF0_SCL_IN)                                  // SCL is pulled LO 
         {
            SysTimer_TWITO_Start();
            while(!TWI_IF0_SCL_IN)                            // while LO
            {
               if (!SysTimer_TWITO_Test())
               {
                  SysTimer_TWITO_End();                       // Stop Timer
                  return(TWIDRV_Status_NACK);                 // Timeout          
               }
            }   
            SysTimer_TWITO_End();                             // Stop Timer
            TWIDelayH_IF0();
         }
        #endif
        if (TWI_IF0_SDA_IN) di|=0x01;
        TWI_IF0_SCL_LO;
        TWI___Delay_L_IF0;
    }
    *data=di;
    return(TWIDRV_Status_Done);
}




unsigned char TWI_Master_ReadByteACK_IF0(void)
{
    TWI_IF0_SDA_LO;             // ACK
    TWI___Delay_L_IF0;
    TWI_IF0_SCL_HI;
    TWI___Delay_H_IF0;
    #ifdef TWI_IF0_CLKStrectch
    if (!TWI_IF0_SCL_IN)                                  // SCL is pulled LO 
     {
        SysTimer_TWITO_Start();
        while(!TWI_IF0_SCL_IN)                            // while LO
        {
           if (!SysTimer_TWITO_Test())
           {
              SysTimer_TWITO_End();                       // Stop Timer
              return(TWIDRV_Status_NACK);                 // Timeout          
           }
        }   
        SysTimer_TWITO_End();                             // Stop Timer
        TWIDelayH_IF0();
     }
    #endif
    TWI_IF0_SCL_LO;
    return(TWIDRV_Status_Done);
}
unsigned char TWI_Master_ReadByteNACK_IF0(void)
{
    TWI_IF0_SDA_HI;             // NACK
    TWI___Delay_L_IF0;
    TWI_IF0_SCL_HI;
    TWI___Delay_H_IF0;
    #ifdef TWI_IF0_CLKStrectch
    if (!TWI_IF0_SCL_IN)                                  // SCL is pulled LO 
     {
        SysTimer_TWITO_Start();
        while(!TWI_IF0_SCL_IN)                            // while LO
        {
           if (!SysTimer_TWITO_Test())
           {
              SysTimer_TWITO_End();                       // Stop Timer
              return(TWIDRV_Status_NACK);                 // Timeout          
           }
        }   
        SysTimer_TWITO_End();                             // Stop Timer
        TWIDelayH_IF0();
     }
    #endif
    TWI_IF0_SCL_LO;
    TWI___Delay_L_IF0;
    TWI_IF0_SDA_LO;
    return(TWIDRV_Status_Done);
}
/****************************************************************************/
unsigned char TWI_Master_ReadByte_IF1(unsigned char *data)
{
  unsigned char cnt;
  unsigned char di;
  TWI_IF1_SDA_HI;
  TWI_IF1_SCL_LO;
  
  
  
  #ifndef TWI_IF1_Highspeed

  /*************** Clockstretch M1 ***************/
  #ifdef TWI_IF1_CLKStrectch_M1
    for (cnt=0;cnt<8;cnt++)
    {
        di=di << 1;
        TWIDelay_Bit(1);
        TWI_IF1_SCL_HI;
        TWIDelay_IF1();         // delay 1x Bit
        if (TWI_IF1_SDA_IN) di|=0x01;
        TWI_IF1_SCL_LO;
        TWIDelayL_IF1();        // delay 1x Bit  
    }
  
  #endif    
  #ifndef TWI_IF1_CLKStrectch_M1
  for (cnt=0;cnt<8;cnt++)
    {
        di=di << 1;
        TWIDelay_Bit(1);
        TWI_IF1_SCL_HI;
        TWIDelay_IF1();         // delay 1x Bit
        if (TWI_IF1_SDA_IN) di|=0x01;
        TWI_IF1_SCL_LO;
        TWIDelayL_IF1();        // delay 1x Bit  
    }
  #endif  
  
  #endif
  
  
  
  
  
  #ifdef TWI_IF1_Highspeed
    for (cnt=0;cnt<8;cnt++)
    {
        di=di << 1;
        TWI_IF1_SCL_HI;
        TWI_Delay_HS;               // delay 1x Bit
        #ifdef TWI_IF1_CLKStrectch
        if (!TWI_IF1_SCL_IN)                                  // SCL is pulled LO 
         {
            SysTimer_TWITO_Start();
            while(!TWI_IF1_SCL_IN)                            // while LO
            {
               if (!SysTimer_TWITO_Test())
               {
                  SysTimer_TWITO_End();                       // Stop Timer
                  return(TWIDRV_Status_NACK);                 // Timeout          
               }
            }   
            SysTimer_TWITO_End();                             // Stop Timer
            TWI_Delay_HS;
         }
        #endif
        if (TWI_IF1_SDA_IN) di|=0x01;
        TWI_IF1_SCL_LO;
        TWI_Delay_HS;         // delay 1x Bit  
    }
    
  #endif  
  *data=di;
  return(TWIDRV_Status_Done);
}





unsigned char TWI_Master_ReadByteACK_IF1(void)
{
    #ifndef TWI_IF1_Highspeed
    
    #ifdef TWI_IF1_LONGACK
    TWIDelayLONGACK_IF1();
    #endif
    
    TWI_IF1_SDA_LO;             // ACK
    TWIDelay_IF1();
    TWI_IF1_SCL_HI;
    TWIDelay_IF1();
    
    #ifdef TWI_IF1_CLKStrectch
    if (!TWI_IF1_SCL_IN)                                  // SCL is pulled LO 
     {
        SysTimer_TWITO_Start();
        while(!TWI_IF1_SCL_IN)                            // while LO
        {
           if (!SysTimer_TWITO_Test())
           {
              SysTimer_TWITO_End();                       // Stop Timer
              return(TWIDRV_Status_NACK);                 // Timeout          
           }
        }   
        SysTimer_TWITO_End();                             // Stop Timer
        TWIDelay_IF1();
     }
    #endif
    
    
    
    TWI_IF1_SCL_LO;
    #endif
    
    
    
    #ifdef TWI_IF1_Highspeed
    TWI_IF1_SDA_LO;             // ACK
    TWI_Delay_HS;
    TWI_IF1_SCL_HI;
    TWI_Delay_HS;
    #ifdef TWI_IF1_CLKStrectch
    if (!TWI_IF1_SCL_IN)                                  // SCL is pulled LO 
      {
        SysTimer_TWITO_Start();
        while(!TWI_IF1_SCL_IN)                            // while LO
        {
           if (!SysTimer_TWITO_Test())
           {
              SysTimer_TWITO_End();                       // Stop Timer
              return(TWIDRV_Status_NACK);                 // Timeout          
           }
        }   
        SysTimer_TWITO_End();                             // Stop Timer
        TWI_Delay_HS;
     }
    #endif
    TWI_IF1_SCL_LO;
    #endif
    return(0);
}
unsigned char TWI_Master_ReadByteNACK_IF1(void)
{
    #ifndef TWI_IF1_Highspeed
    #ifdef TWI_IF1_LONGACK
    TWIDelayLONGACK_IF1();
    #endif
    
    TWI_IF1_SDA_HI;             // NACK
    TWIDelay_IF1();
    TWI_IF1_SCL_HI;
    TWIDelay_IF1();
    #ifdef TWI_IF1_CLKStrectch
    if (!TWI_IF1_SCL_IN)                                  // SCL is pulled LO 
     {
        SysTimer_TWITO_Start();
        while(!TWI_IF1_SCL_IN)                            // while LO
        {
           if (!SysTimer_TWITO_Test())
           {
              SysTimer_TWITO_End();                       // Stop Timer
              return(TWIDRV_Status_NACK);                 // Timeout          
           }
        }   
        SysTimer_TWITO_End();                             // Stop Timer
        TWIDelay_IF1();
     }
    #endif
    TWI_IF1_SCL_LO;
    #endif
    #ifdef TWI_IF1_Highspeed
    TWI_IF1_SDA_HI;             // NACK
    TWI_Delay_HS;
    TWI_IF1_SCL_HI;
    TWI_Delay_HS;
    #ifdef TWI_IF1_CLKStrectch
    if (!TWI_IF1_SCL_IN)                                  // SCL is pulled LO 
     {
        SysTimer_TWITO_Start();
        while(!TWI_IF1_SCL_IN)                            // while LO
        {
           if (!SysTimer_TWITO_Test())
           {
              SysTimer_TWITO_End();                       // Stop Timer
              return(TWIDRV_Status_NACK);                 // Timeout          
           }
        }   
        SysTimer_TWITO_End();                             // Stop Timer
        TWI_Delay_HS;
     }
    #endif
    TWI_IF1_SCL_LO;   
    #endif
    return(0);
}
/****************************************************************************/
/*** Write for IF0 & IF 1                                                 ***/
/****************************************************************************/
unsigned char TWI_Master_WriteByte_IF0(unsigned char data)
{
  unsigned char cnt;
  unsigned char ACK;
  TWI_IF0_SDA_LO;
  TWI_IF0_SCL_LO;
  for (cnt=0;cnt<8;cnt++)
    {
        if (data & 0x80) 
   	      {TWI_IF0_SDA_HI;}       // SDA HI
	        else
	      {TWI_IF0_SDA_LO;}       // SDA LO 

        TWI___Delay_Bit_IF0;
        TWI_IF0_SCL_HI;           // SCL HI
        TWI___Delay_H_IF0;
        #ifdef TWI_IF0_CLKStrectch
         if (!TWI_IF0_SCL_IN)                                 // SCL is pulled LO 
         {
            SysTimer_TWITO_Start();
            while(!TWI_IF0_SCL_IN)                            // while LO
            {
               if (!SysTimer_TWITO_Test())
               {
                  SysTimer_TWITO_End();                       // Stop Timer
                  return(TWIDRV_Status_NACK);                 // Timeout          
               }
            }   
            SysTimer_TWITO_End();                             // Stop Timer
            TWI___Delay_H_IF0;
         }
        #endif
        
        TWI_IF0_SCL_LO;             // SCL LO
        data=data << 1;
        TWI___Delay_L_IF0;       
        
    }
    
    
    TWI_IF0_SDA_HI;                                    // SDA HI
    TWI___Delay_L_IF0;
    
    
    TWI_IF0_SCL_HI;                                     // SCL HI
    TWI___Delay_H_IF0;
 
    
    /* ACK Pulse detect */
    #ifdef TWI_IF0_CLKStrectch
    if (!TWI_IF0_SCL_IN)                        // SCL is pulled LO 
    {
       SysTimer_TWITO_Start();
       while(!TWI_IF0_SCL_IN)                   // while LO
         {
           if (!SysTimer_TWITO_Test())
            {
              SysTimer_TWITO_End();             // Stop Timer
              return(TWIDRV_Status_NACK);       // Timeout          
            }
         }   
       SysTimer_TWITO_End();                    // Stop Timer
       TWI___Delay_H_IF0;
    }
    #endif
    
    ACK=1;
    if (!TWI_IF0_SDA_IN) ACK=0;
    TWI_IF0_SCL_LO;
    TWI___Delay_L_IF0;
    TWI_IF0_SDA_LO;
    if (ACK==0)  return(TWIDRV_Status_Ready);   // ACK = OK
    return(TWIDRV_Status_NACK);                 // ACK = None  
  
}

unsigned char TWI_Master_WriteByte_IF1(unsigned char data)
{
  unsigned char cnt;
  unsigned char ACK;
  TWI_IF1_SDA_LO;
  TWI_IF1_SCL_LO;
  #ifndef TWI_IF1_Highspeed
  
  
  /*************** Clockstretch M1 ***************/
  #ifdef TWI_IF1_CLKStrectch_M1

  for (cnt=0;cnt<7;cnt++)         // only 7 bit  
  {
    TWIDelayL_IF1();               //  |--| Delay
    if (data & 0x80) 
        {TWI_IF1_SDA_HI;}         // SDA HI
	    else
	    {TWI_IF1_SDA_LO;}         // SDA LO 
     TWIDelay_Bit(1);             //  |-| Delay
     TWI_IF1_SCL_HI;              // SCL HI
     TWIDelay_IF1();              //  |--| Delay
     TWI_IF1_SCL_LO;              // SCL LO
     data=data << 1;
  }

  /* Bit #8 */
  TWIDelayL_IF1();               //  |--| Delay
  if (data & 0x80) 
        {TWI_IF1_SDA_HI;}       // SDA HI
	    else
	    {TWI_IF1_SDA_LO;}       // SDA LO 
  TWIDelay_Bit(1);              //  |-| Delay
  TWI_IF1_SCL_HI;               // SCL HI
  if (TWI_CHECK_ClockStretch_IF1() == TWIDRV_Status_NACK) return(TWIDRV_Status_NACK);
  return(TWIDRV_Status_Ready);
  #endif

  
  /*************** Clockstretch M2 ***************/
  #ifdef TWI_IF1_CLKStrectch_M2
  for (cnt=0;cnt<7;cnt++)
  {
    TWIDelayL_IF1();               //  |--| Delay
    if (data & 0x80) 
        {TWI_IF1_SDA_HI;}         // SDA HI
	    else
	    {TWI_IF1_SDA_LO;}         // SDA LO 
     TWIDelay_Bit(1);             //  |-| Delay
     TWI_IF1_SCL_HI;              // SCL HI
     TWIDelay_IF1();              //  |--| Delay
     TWI_IF1_SCL_LO;              // SCL LO
     data=data << 1;
  }
  /* Bit #8 */
  TWIDelay_IF1();                 //  |--| Delay
  if (data & 0x80) 
        {TWI_IF1_SDA_HI;}         // SDA HI
	    else
	    {TWI_IF1_SDA_LO;}         // SDA LO 
  TWIDelay_Bit(1);                //  |-| Delay
  TWI_IF1_SCL_HI;                 // SCL HI
   
  TWI_CHECK_ClockStretchM2_IF1(); // check if Slave pulls low, then finish
  TWIDelayL_IF1();
  TWI_IF1_SDA_HI;
  #ifdef TWI_IF1_LONGACK
  TWIDelayLONGACK_IF1();
  #endif
  #ifndef TWI_IF1_LONGACK
  TWIDelay_IF1();
  #endif  
  
  TWI_IF1_SCL_HI;
  TWIDelay_IF1(); 
  ACK=1;
  if (!TWI_IF1_SDA_IN) ACK=0;
  TWI_IF1_SCL_LO;
  TWIDelayL_IF1();
  TWI_IF1_SDA_LO;
  if (ACK==0)  return(TWIDRV_Status_Ready);   // ACK = OK
  return(TWIDRV_Status_NACK);                 // ACK = None
  
  #endif

  /*************** Standard ***************/
  #ifndef TWI_IF1_CLKStrectch_M1
  #ifndef TWI_IF1_CLKStrectch_M2
  for (cnt=0;cnt<8;cnt++)
  {
    TWIDelay_IF1();               //  |--| Delay
    if (data & 0x80) 
        {TWI_IF1_SDA_HI;}         // SDA HI
	    else
	    {TWI_IF1_SDA_LO;}         // SDA LO 
     TWIDelay_Bit(1);             //  |-| Delay
     TWI_IF1_SCL_HI;              // SCL HI
     TWIDelay_IF1();              //  |--| Delay
     TWI_IF1_SCL_LO;              // SCL LO
     data=data << 1;
  }
  
  TWIDelayL_IF1();
  TWI_IF1_SDA_HI;
  #ifdef TWI_IF1_LONGACK
  TWIDelayLONGACK_IF1();
  #endif
  #ifndef TWI_IF1_LONGACK
  TWIDelay_IF1();
  #endif  
  
  TWI_IF1_SCL_HI;
  TWIDelay_IF1(); 
  ACK=1;
  if (!TWI_IF1_SDA_IN) ACK=0;
  TWI_IF1_SCL_LO;
  TWIDelayL_IF1();
  TWI_IF1_SDA_LO;
  if (ACK==0)  return(TWIDRV_Status_Ready);   // ACK = OK
  return(TWIDRV_Status_NACK);                 // ACK = None
  #endif
  #endif
  
  #endif


  /*************** High Speed ***************/
  #ifdef TWI_IF1_Highspeed
   for (cnt=0;cnt<8;cnt++)
    {
        if (data & 0x80) 
   	      {TWI_IF1_SDA_HI;}
	        else
	        {TWI_IF1_SDA_LO;}
        TWI_IF1_SCL_HI;
        TWI_Delay_HS;
        TWI_IF1_SCL_LO;
        data=data << 1;
        TWI_Delay_HS;
        TWI_Delay_HS;
    }
    TWI_IF1_SDA_HI;
    TWI_IF1_SCL_HI;
    TWI_Delay_HS;
    TWI_Delay_HS;
    ACK=1;
    if (!TWI_IF1_SDA_IN) ACK=0;
    TWI_IF1_SCL_LO;
    TWI_Delay_HS;
    TWI_Delay_HS;
   
   
    if (ACK==0)  return(TWIDRV_Status_Ready);   // ACK = OK
    return(TWIDRV_Status_NACK);                 // ACK = None  
  #endif
}

void TWIDelay_10ms(void)
{
unsigned char c;
for (c=0;c<10;c++)  SysTimer_Delay_cylces(cnst_TWI_Delay_10ms);
}

/***********************************************************************/
// internal functions below

void TWIDelayH_IF0(void)
{
 if (TWI_BaudrateSet[0]==cnst_TWI_Baudrate_25k)  TWIDelayH_25kHz();
 if (TWI_BaudrateSet[0]==cnst_TWI_Baudrate_100k)  TWIDelayH_100kHz();
 if (TWI_BaudrateSet[0]==cnst_TWI_Baudrate_400k)  TWIDelayH_400kHz();
}
void TWIDelayL_IF0(void)
{
 if (TWI_BaudrateSet[0]==cnst_TWI_Baudrate_25k)  TWIDelayL_25kHz();
 if (TWI_BaudrateSet[0]==cnst_TWI_Baudrate_100k)  TWIDelayL_100kHz();
 if (TWI_BaudrateSet[0]==cnst_TWI_Baudrate_400k)  TWIDelayL_400kHz();
}
void TWIDelayInterByte_IF0(void)
{
 if (TWI_BaudrateSet[0]==cnst_TWI_Baudrate_25k)   TWIDelayIB_25kHz();
 if (TWI_BaudrateSet[0]==cnst_TWI_Baudrate_100k)  TWIDelayIB_100kHz();
 if (TWI_BaudrateSet[0]==cnst_TWI_Baudrate_400k)  TWIDelayIB_400kHz();
}


void TWIDelay_IF1(void)
{
 if (TWI_BaudrateSet[1]==cnst_TWI_Baudrate_25k)  TWIDelayH_25kHz();
 if (TWI_BaudrateSet[1]==cnst_TWI_Baudrate_100k)  TWIDelayH_100kHz();
 if (TWI_BaudrateSet[1]==cnst_TWI_Baudrate_400k)  TWIDelayH_400kHz();
}
void TWIDelayL_IF1(void)
{
 if (TWI_BaudrateSet[1]==cnst_TWI_Baudrate_25k)  TWIDelayL_25kHz();
 if (TWI_BaudrateSet[1]==cnst_TWI_Baudrate_100k)  TWIDelayL_100kHz();
 if (TWI_BaudrateSet[1]==cnst_TWI_Baudrate_400k)  TWIDelayL_400kHz();
}

void TWIDelayLONGACK_IF1(void)
{
 if (TWI_BaudrateSet[1]==cnst_TWI_Baudrate_25k)   TWIDelayLONGACK_25kHz();
 if (TWI_BaudrateSet[1]==cnst_TWI_Baudrate_100k)  TWIDelayLONGACK_100kHz();
 if (TWI_BaudrateSet[1]==cnst_TWI_Baudrate_400k)  TWIDelayLONGACK_400kHz();
}






// Bit Low/Hi Time Delay
void TWIDelayH_25kHz(void)
{
SysTimer_Delay_cylces(cnst_TWI_DelayH_25kHz);
}
void TWIDelayL_25kHz(void)
{
SysTimer_Delay_cylces(cnst_TWI_DelayL_25kHz);
}


void TWIDelayH_400kHz(void)
{
//SysTimer_Delay_cylces(cnst_TWI_DelayH_400kHz);
_TWIdelay_loop(cnst_TWI_DelayH_400kHz);
}
void TWIDelayL_400kHz(void)
{
//SysTimer_Delay_cylces(cnst_TWI_DelayH_400kHz);
_TWIdelay_loop(cnst_TWI_DelayL_400kHz);
}


// Bit Low/Hi Time Delay
void TWIDelayH_100kHz(void)
{
_TWIdelay_loop(cnst_TWI_DelayH_100kHz);
}
void TWIDelayL_100kHz(void)
{
_TWIdelay_loop(cnst_TWI_DelayL_100kHz);
} 


void TWIDelayLONGACK_25kHz(void)
{
_TWIdelay_loop(cnst_TWI_DelayLACK_25kHz);
_TWIdelay_loop(cnst_TWI_DelayLACK_25kHz);
}
void TWIDelayLONGACK_100kHz(void)
{
_TWIdelay_loop(cnst_TWI_DelayLACK_100kHz);
_TWIdelay_loop(cnst_TWI_DelayLACK_100kHz);
}
void TWIDelayLONGACK_400kHz(void)
{
_TWIdelay_loop(cnst_TWI_DelayLACK_400kHz);
_TWIdelay_loop(cnst_TWI_DelayLACK_400kHz);
}





// Inter Byte Delay
void TWIDelayIB_25kHz(void)
{
SysTimer_Delay_cylces(cnst_TWI_DelayIB_25kHz);
}
void TWIDelayIB_100kHz(void)
{
SysTimer_Delay_cylces(cnst_TWI_DelayIB_100kHz);
}
void TWIDelayIB_400kHz(void)
{
SysTimer_Delay_cylces(cnst_TWI_DelayIB_400kHz);
}




void TWIDelay_Bit(unsigned char Interface)
{
 if (TWI_BaudrateSet[Interface]==cnst_TWI_Baudrate_400k)  
  {
    _TWIdelay_loop(cnst_TWI_Delay_Bit400k);
    return;
  }  
 if (TWI_BaudrateSet[Interface]==cnst_TWI_Baudrate_100k) _TWIdelay_loop(cnst_TWI_Delay_Bit100k);
 if (TWI_BaudrateSet[Interface]==cnst_TWI_Baudrate_25k)  _TWIdelay_loop(cnst_TWI_Delay_Bit100k);
}

/***************************************************************************/
unsigned char TWI_CHECK_ClockStretch_IF1(void)
{
        
        unsigned char exitloop;
        unsigned char ACK;
        unsigned int  delaycnt;
        
        #ifdef TWI_IF1_CLKStrectch_M1
        // SCL=HI has just started
        // -> so delay until max. HI and poll Pin
        exitloop=0;
        delaycnt=cnst_TWI_DelayH_CLKStretch_M1WRITE;     // Load maximum High Timer
        while(!exitloop)
        {
            if (!TWI_IF1_SCL_IN)                    // SCL is pulled LO
            {
                exitloop=1;     // -> CLK Stretching
            }
            if (!delaycnt)
            {
                exitloop=2;     // -> NO CLK Stretching but HighTime reached
            }
            delaycnt--;
        }
       
       
       /**  CLK Stretching active **/ 
       if (exitloop==1)         
       {
            // wait while LO or timeout
            SysTimer_TWITO_Start();                           // Start timeout
            while(!TWI_IF1_SCL_IN)                            // while LO
            {
               if (!SysTimer_TWITO_Test())
               {
                  SysTimer_TWITO_End();                       // Stop Timer
                  return(TWIDRV_Status_NACK);                 // Timeout          
               }
            }   
            SysTimer_TWITO_End();                             // Stop Timer

            // let SCL HI for standard Time, this is ACK pulse
            TWIDelay_IF1();
            ACK=1;
            if (!TWI_IF1_SDA_IN) ACK=0;
            TWI_IF1_SCL_LO;
            TWIDelay_IF1();
            TWI_IF1_SDA_LO;
            if (ACK==0)  return(TWIDRV_Status_Ready);   // ACK = OK
            return(TWIDRV_Status_NACK);                 // ACK = None  

       }
        /**  CLK Stretching inactive **/ 
       if (exitloop==2)         
       {
            // let SCL HI for standard Time, 
            TWIDelay_IF1();
            TWI_IF1_SCL_LO;

            // start ACK pulse
            TWIDelay_Bit(1);
            TWI_IF1_SDA_HI;
            TWIDelay_IF1();
            TWI_IF1_SCL_HI;
            
            ACK=1;
            if (!TWI_IF1_SDA_IN) ACK=0;
            TWI_IF1_SCL_LO;
            TWIDelay_IF1();
            TWI_IF1_SDA_LO;
            if (ACK==0)  return(TWIDRV_Status_Ready);   // ACK = OK
            return(TWIDRV_Status_NACK);                 // ACK = None  
       }        
       #endif

}
/***************************************************************************/
unsigned char TWI_CHECK_ClockStretchREAD_IF1(void)
{
        
        unsigned char exitloop;
        unsigned char ACK;
        unsigned int  delaycnt;
        
        #ifdef TWI_IF1_CLKStrectch_M1
        // SCL=HI has just started
        // -> so delay until max. HI and poll Pin
        exitloop=0;
        delaycnt=cnst_TWI_DelayH_CLKStretch_M1READ;  // Load maximum High Timer
        while(!exitloop)
        {
            if (!TWI_IF1_SCL_IN)                    // SCL is pulled LO
            {
                TWI_IF1_SCL_LO;
                exitloop=1;     // -> CLK Stretching
            }
            if (!delaycnt)
            {
                exitloop=2;     // -> NO CLK Stretching but HighTime reached
            }
            delaycnt--;
        }
        #endif
       
      

}

unsigned char TWI_CHECK_ClockStretchM2_IF1(void)
{       
        #ifdef TWI_IF1_CLKStrectch_M2
        unsigned char exitloop;
        unsigned char ACK;
        unsigned int  delaycnt;
        
        // SCL=HI has just started
        // -> so delay until max. HI and poll Pin
        exitloop=0;
        delaycnt=cnst_TWI_DelayH_CLKStretch_M1WRITE;     // Load maximum High Timer
        while(!exitloop)
        {
            if (!TWI_IF1_SCL_IN)                    // SCL is pulled LO
            {
                exitloop=1;     // -> CLK Stretching
                TWI_IF1_SCL_LO; // -> Set SCL=LO
            }
            if (!delaycnt)
            {
                exitloop=2;     // -> NO CLK Stretching but HighTime reached
                TWI_IF1_SCL_LO; // -> Set SCL=LO
            }
            delaycnt--;
        }
        #endif
}        



// internal delay function. 7  + n*3 cycles  -Os: @32MHz -> 94ns/loop + 220ns call
// internal delay function. 26 + n*3 cycles  -O0: @32MHz -> 94ns/loop + 812ns call
void
_TWIdelay_loop(uint8_t __count)
{
	__asm__ volatile (
		"1: dec %0" "\n\t"
		"brne 1b"
		: "=r" (__count)
		: "0" (__count)
	);
}


