
/**********************************************************************/

/*****************************************************************************/
#ifndef TWI_MP_H
#define TWI_MP_H

#include "avr_compiler.h"


#define cnst_TWI_Baudrate_100k  1
#define cnst_TWI_Baudrate_400k  2
#define cnst_TWI_Baudrate_1M    0
#define cnst_TWI_Baudrate_25k   4

// 100kHz = ~ 82kHz eff
// 400kHz = ~ 189kHz eff
// 1MHz   = ~ 670kHz eff

/**********************************/
//****** Interface Settings ******//
/**********************************/

// define a looong delay before ACK (see delay for constant)
#define TWI_IF1_LONGACK


// define if clock strecthing is recognized or not
//#define TWI_IF0_CLKStrectch   1
//#define TWI_IF1_CLKStrectch   1
//#define TWI_IF1_CLKStrectch_M1   1
#define TWI_IF1_CLKStrectch_M2   1

// define if >= 1MBit is used
//#define TWI_IF1_Highspeed       1
#define TWI_IF0_Highspeed       1



//*** delay settings ***//

// internal delay function: 7  + n*3 cycles  -Os: @32MHz -> 94ns/loop + 220ns call
// internal delay function: 26 + n*3 cycles  -O0: @32MHz -> 94ns/loop + 812ns call

// These prototypes are necessary for the following defines....

#define TWI_Delay_HS_Bit   _TWIdelay_loop(2)
#define TWI_Delay_HS_H     _TWIdelay_loop(7)
#define TWI_Delay_HS_L     _TWIdelay_loop(1)
void TWIDelayH_IF0(void);
void TWIDelayL_IF0(void);
void TWIDelay_Bit(unsigned char Interface);

#ifdef  TWI_IF0_Highspeed 
   #define TWI___Delay_Bit_IF0   TWI_Delay_HS_Bit       
   #define TWI___Delay_H_IF0     TWI_Delay_HS_H
   #define TWI___Delay_L_IF0     TWI_Delay_HS_L
#endif                   
#ifndef  TWI_IF0_Highspeed 
   #define TWI___Delay_Bit_IF0   TWIDelay_Bit(0)
   #define TWI___Delay_H_IF0     TWIDelayH_IF0()
   #define TWI___Delay_L_IF0     TWIDelayL_IF0()          
#endif




// delay constants for 32MHz
// Hi Time
#define cnst_TWI_DelayH_25kHz    800
//#define cnst_TWI_DelayH_100kHz   138
#define cnst_TWI_DelayH_100kHz   30
#define cnst_TWI_DelayH_400kHz   14
#define cnst_TWI_DelayH_1MHz     0

#define cnst_TWI_DelayH_CLKStretch_M1WRITE  40
#define cnst_TWI_DelayH_CLKStretch_M1READ   200

// Lo Time
#define cnst_TWI_DelayL_25kHz     600
//#define cnst_TWI_DelayL_100kHz   20
#define cnst_TWI_DelayL_100kHz    10
#define cnst_TWI_DelayL_400kHz    1
#define cnst_TWI_DelayL_1MHz      0

// Long ACK timing
#define cnst_TWI_DelayLACK_100kHz 156   /* value givven is the half */
#define cnst_TWI_DelayLACK_400kHz 156   /* value givven is the half */
#define cnst_TWI_DelayLACK_25kHz  156   /* value givven is the half */

// Bit change  Time
#define cnst_TWI_Delay_Bit100k  20
#define cnst_TWI_Delay_Bit400k  3
#define cnst_TWI_Delay_10ms     33000

// Inter Byte Delay
#define cnst_TWI_DelayIB_25kHz   400
#define cnst_TWI_DelayIB_100kHz  400
#define cnst_TWI_DelayIB_400kHz  400



// Delay:
// Delay for 100kHz is in Clockcycles   (35 min)
// Delay for 400kHz is in 3xClockcycles (1 min)


// Pin defs
#define TWI_IF0_SDA_Bit   0
#define TWI_IF0_SCL_Bit   1
#define TWI_IF1_SDA_Bit   0
#define TWI_IF1_SCL_Bit   1

#define TWI_IF0_SDA_Init    PORTE.OUT&=~(1<<TWI_IF0_SDA_Bit)
#define TWI_IF0_SDA_LO    	PORTE.DIR|=(1<<TWI_IF0_SDA_Bit)
#define TWI_IF0_SDA_HI    	PORTE.DIR&=~(1<<TWI_IF0_SDA_Bit)
#define TWI_IF0_SDA_IN    	(PORTE.IN&(1<<TWI_IF0_SDA_Bit))

#define TWI_IF0_SCL_Init    PORTE.OUT&=~(1<<TWI_IF0_SCL_Bit)
#define TWI_IF0_SCL_LO    	PORTE.DIR|=(1<<TWI_IF0_SCL_Bit)
#define TWI_IF0_SCL_HI    	PORTE.DIR&=~(1<<TWI_IF0_SCL_Bit)
#define TWI_IF0_SCL_IN    	(PORTE.IN&(1<<TWI_IF0_SCL_Bit))

#define TWI_IF1_SDA_Init    PORTC.OUTCLR=(1<<TWI_IF1_SDA_Bit)
#define TWI_IF1_SDA_LO    	PORTC.DIRSET=(1<<TWI_IF1_SDA_Bit)
#define TWI_IF1_SDA_HI    	PORTC.DIRCLR=(1<<TWI_IF1_SDA_Bit)
#define TWI_IF1_SDA_IN    	(PORTC.IN&(1<<TWI_IF1_SDA_Bit))

#define TWI_IF1_SCL_Init    PORTC.OUTCLR=(1<<TWI_IF1_SCL_Bit)
#define TWI_IF1_SCL_LO    	PORTC.DIRSET=(1<<TWI_IF1_SCL_Bit)
#define TWI_IF1_SCL_HI    	PORTC.DIRCLR=(1<<TWI_IF1_SCL_Bit)
#define TWI_IF1_SCL_IN    	(PORTC.IN&(1<<TWI_IF1_SCL_Bit))




/*! Transaction status defines. */
#define TWIM_STATUS_READY              0
#define TWIM_STATUS_BUSY               1

#define TWIDRV_Status_BUSY              1
#define TWIDRV_Status_Done              0
#define TWIDRV_Status_Ready             0
#define TWIDRV_Status_NACK              3
#define TWIDRV_Status_Timeout           2
#define TWIDRV_Status_Error             4




volatile unsigned char TWI_BaudrateSet[8];

void TWI_MasterInit(unsigned char InterFaceNum,unsigned char TWI_Baudrate);
unsigned char TWI_MasterWriteRead(unsigned char InterFaceNum,
                                  unsigned char Adress,
                                  unsigned char *BufferWR,
                                  unsigned char *BufferRD,
                                  unsigned char WRNum,
                                  unsigned char RDNum);
unsigned char TWI_Master_CheckDeviceACK(unsigned char InterFaceNum,unsigned char Adress);


void TWI_Master_START_IF0(void);
void TWI_Master_START_IF1(void);
void TWI_Master_STOP_IF0(void);
void TWI_Master_STOP_IF1(void);
unsigned char TWI_Master_ReadByte_IF0(unsigned char *data);
unsigned char TWI_Master_ReadByteACK_IF0(void);
unsigned char TWI_Master_ReadByteNACK_IF0(void);
unsigned char TWI_Master_ReadByte_IF1(unsigned char *data);
unsigned char TWI_Master_ReadByteACK_IF1(void);
unsigned char TWI_Master_ReadByteNACK_IF1(void);
unsigned char TWI_Master_WriteByte_IF1(unsigned char data);                                  
unsigned char TWI_Master_WriteByte_IF0(unsigned char data);


void TWIDelayInterByte_IF0(void);

void TWIDelay_IF1(void);
void TWIDelayL_IF1(void);
void TWIDelayH_100kHz(void);
void TWIDelayH_25kHz(void);
void TWIDelayH_400kHz(void);
void TWIDelayL_100kHz(void);
void TWIDelayL_25kHz(void);
void TWIDelayL_400kHz(void);

void TWIDelayLONGACK_IF1(void);
void TWIDelayLONGACK_25kHz(void);
void TWIDelayLONGACK_100kHz(void);
void TWIDelayLONGACK_400kHz(void);

void TWIDelayIB_25kHz(void);
void TWIDelayIB_100kHz(void);
void TWIDelayIB_400kHz(void);

void TWIDelay_10ms(void);
void TWIDelay_1MHz(void);


unsigned char TWI_CHECK_ClockStretch_IF1(void);
unsigned char TWI_CHECK_ClockStretchM2_IF1(void);


void
_TWIdelay_loop(uint8_t __count);
#endif /* TWI_MASTER_DRIVER_H */
