
#include "CLI.h"
#include "usart_comm.h"
#include "dev_powerDist.h"
#include "devices.h"
#include "COMM485.h"
#include "EXT_I2C.h"
#include "EEP_adresses.h"
#include <avr\io.h>
#include <stdio.h>

/* functions use global DIB/PIB for data transfer */
/* functions use I2C-IO to reach PowerDist */
/* functions return Error value as uchar */
/***********************************************************/
/*                                                        */
/***********************************************************/
unsigned char PowerDist_WRITE(unsigned char *TxBuffer)
{
 	  unsigned char ret;
 	  unsigned char Err;
   	unsigned char ADR1,ADR2,ADR3,P0,P1,PB1,PB2,PB3,PB1ON,PB2ON,PB3ON,PB3valid;
   	unsigned char result;

    PB3valid=1; // set valid 

    GlobalPIB.CMD=CLI_PCMD_S_PowerDist;
    DEV_4BYTE_GetDIB(GlobalPIB.DeviceNum);     // Get DIB(num)
    
    PowerDist_MAP_PIBval_2_Portbits(&PB1ON,&PB2ON,&PB3ON);  // get ON or OFF
    PowerDist_GET_I2Cadresses_from_DIB(&ADR1,&ADR2,&ADR3);  // get Adresses
    PowerDist_GET_Portbits_from_DIB(&PB1,&PB2,&PB3);
    
    if ((PB3==PB2) && (ADR2==ADR3))                         // check if B2==B3
    {
        PB3valid=0;                                         // PB3 invalid         
    }    
    
    result=0;
    Err=EXT_I2C_Error_NONE;
    
    ret=EXT_I2C_ReadPorts_PCA9555(ADR1,&P0,&P1);// get bits on Adress    
    if (ret != EXT_I2C_Error_NONE) Err=ret;
    if (PB1 > 7)
     {
        if (PB1ON)  P1=SETBITinVar_uchar(P1,PB1-8);
        if (!PB1ON) P1=CLRBITinVar_uchar(P1,PB1-8);
     }
     else
     {
        if (PB1ON)  P0=SETBITinVar_uchar(P0,PB1);
        if (!PB1ON) P0=CLRBITinVar_uchar(P0,PB1);
     }
    ret=EXT_I2C_WritePorts_PCA9555(ADR1,P0,P1);   // write back
    if (ret != EXT_I2C_Error_NONE) Err=ret;
    
    
    ret=EXT_I2C_ReadPorts_PCA9555(ADR2,&P0,&P1);// get bits on Adress    
    if (ret != EXT_I2C_Error_NONE) Err=ret;
    if (PB2 > 7)
     {
        if (PB2ON)  P1=SETBITinVar_uchar(P1,PB2-8);
        if (!PB2ON) P1=CLRBITinVar_uchar(P1,PB2-8);
     }
     else
     {
        if (PB2ON)  P0=SETBITinVar_uchar(P0,PB2);
        if (!PB2ON) P0=CLRBITinVar_uchar(P0,PB2);
     }
    ret=EXT_I2C_WritePorts_PCA9555(ADR2,P0,P1);   // write back
    if (ret != EXT_I2C_Error_NONE) Err=ret;
    
    
    ret=EXT_I2C_ReadPorts_PCA9555(ADR3,&P0,&P1);// get bits on Adress    
    if (ret != EXT_I2C_Error_NONE) Err=ret;
    if (PB3 > 7)
     {
        if ((PB3ON) && (PB3valid))  P1=SETBITinVar_uchar(P1,PB3-8);
        if ((!PB3ON) && (PB3valid)) P1=CLRBITinVar_uchar(P1,PB3-8);
     }
     else
     {
        if ((PB3ON)&& (PB3valid))  P0=SETBITinVar_uchar(P0,PB3);
        if ((!PB3ON)&& (PB3valid)) P0=CLRBITinVar_uchar(P0,PB3);
     }
    if (PB3valid) 
    {
      ret=EXT_I2C_WritePorts_PCA9555(ADR3,P0,P1);   // write back
      if (ret != EXT_I2C_Error_NONE) Err=ret;
    }  
    
    
    GlobalPIB.Param_EE=0;

	if (Err!=EXT_I2C_Error_NONE) 
	{
        return(Err);
	}
	else
	{		
		UCOMM_SendAnswerpacket(TxBuffer);
        return(UCOMM_Error_None);			
	}	


}
/***********************************************************/
/*                                                        */
/***********************************************************/
unsigned char PowerDist_READ(unsigned char *TxBuffer)
{
   	unsigned char ret;
   	unsigned char ADR1,ADR2,ADR3,P0,P1,PB1,PB2,PB3,PB3valid;
   	unsigned char t8,result;
    unsigned char Err;
    
    PB3valid=1;

    GlobalPIB.CMD=CLI_PCMD_S_PowerDist;
    DEV_4BYTE_GetDIB(GlobalPIB.DeviceNum);                  // Get DIB(num)
    
    PowerDist_GET_I2Cadresses_from_DIB(&ADR1,&ADR2,&ADR3);  // get Adresses
    PowerDist_GET_Portbits_from_DIB(&PB1,&PB2,&PB3);
    
    if ((PB3==PB2) && (ADR2==ADR3))                         // check if B2==B3
    {
        PB3valid=0;                                         // PB3 invalid         
    }    
    
    result=0;
    Err=EXT_I2C_Error_NONE;
    
    ret=EXT_I2C_ReadPorts_PCA9555(ADR1,&P0,&P1);                // get bits on Adress
    if (ret != EXT_I2C_Error_NONE) Err=ret;
    if (PB1 > 7)
     {
        t8=IsBitSETinVar_uchar(P1,PB1-8);
     }
     else
     {
        t8=IsBitSETinVar_uchar(P0,PB1);
     }   
    if (t8) result|=(1<<0);

    ret=EXT_I2C_ReadPorts_PCA9555(ADR2,&P0,&P1);                // get bits on Adress
    if (ret != EXT_I2C_Error_NONE) Err=ret;
    if (PB2 > 7)
     {
        t8=IsBitSETinVar_uchar(P1,PB2-8);
     }
     else
     {
        t8=IsBitSETinVar_uchar(P0,PB2);
     }   
    if (t8) result|=(1<<1);

    ret=EXT_I2C_ReadPorts_PCA9555(ADR3,&P0,&P1);                // get bits on Adress
    if (ret != EXT_I2C_Error_NONE) Err=ret;
    if (PB3 > 7)
     {
        t8=IsBitSETinVar_uchar(P1,PB3-8);
     }
     else
     {
        t8=IsBitSETinVar_uchar(P0,PB3);
     }   
    if ((t8) && (PB3valid)) result|=(1<<2);
    GlobalPIB.Param_EE=result;

	if (Err!=EXT_I2C_Error_NONE) 
	{
        return(Err);
	}
	else
	{		
		UCOMM_SendAnswerpacket(TxBuffer);
        return(UCOMM_Error_None);			
	}	
}


/***********************************************************/
/*                                                        */
/***********************************************************/
unsigned char PowerDist_SET(unsigned char *TxBuffer)
{
    unsigned char Error;
    
    Error=UCOMM_Error_None;
    
    GlobalDIB.DeviceNum=GlobalPIB.DeviceNum;
    GlobalDIB.RS485adress=GlobalPIB.Param_EES;  // I2C Adresses Byte 2..0
    GlobalDIB.SUBadress=GlobalPIB.Param_FF;     // Portbit 1 (hi), 2(lo)
    GlobalPIB.Param_GG |= 0x01;                 // set valid (lo)
    GlobalPIB.Param_GG &=~ 0x02;                // set valid (lo)
    GlobalDIB.SUBportbit=GlobalPIB.Param_GG;    // Portbit 3 (hi)
    DEV_4BYTE_WriteDIB(GlobalDIB.DeviceNum);
    if (GlobalDIB.EEPAdress!=0)
    {
        UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
    }
    else
    {
        Error=UCOMM_Error_DevNumInvalid;       // Send Error
    }   
    return(Error);
}    
/***********************************************************/
/*                                                        */
/***********************************************************/
unsigned char PowerDist_GET(unsigned char *TxBuffer)
{
    unsigned char Error;
    unsigned char t8;
    
    Error=UCOMM_Error_None;
    DEV_4BYTE_GetDIB(GlobalPIB.DeviceNum);
    GlobalPIB.Param_EES=GlobalDIB.RS485adress;
    GlobalPIB.Param_FF=GlobalDIB.SUBadress;
    GlobalPIB.Param_GG=GlobalDIB.SUBportbit;
    t8=GlobalDIB.SUBportbit & 0x0F;
    if ((t8) && (t8 < 0x0F))                    // LoNibble >0x0 & <0xF
    {
            UCOMM_SendAnswerpacket_Setup(TxBuffer); // Send Answer
    }
    else
    {
            Error=UCOMM_Error_DevNumInvalid;       // Send Error
    }
    return(Error);
}
    

/***********************************************************/
unsigned char PowerDist_INIT(void)
{

	unsigned int Scancnt;
	unsigned char ADR1,ADR2,ADR3;
	unsigned char ret,Err;
	
	Err=EXT_I2C_Error_NONE;
    GlobalPIB.CMD=CLI_PCMD_S_PowerDist;
	for(Scancnt=1;Scancnt<EEP_MaxNum_PowerDist;Scancnt++)
	{
        GlobalPIB.DeviceNum=Scancnt;
        DEV_4BYTE_GetDIB(Scancnt);
        if (!PowerDist_Test_DIB_invalid())       // valid
		{
            // valid entry found....
            PowerDist_GET_I2Cadresses_from_DIB(&ADR1,&ADR2,&ADR3);  // get Adresses
            ret=EXT_I2C_Init_PCA9555_OUT(ADR1);
            if (ret != EXT_I2C_Error_NONE) Err=ret;
            ret=EXT_I2C_Init_PCA9555_OUT(ADR2);
            if (ret != EXT_I2C_Error_NONE) Err=ret;
            ret=EXT_I2C_Init_PCA9555_OUT(ADR3);
            if (ret != EXT_I2C_Error_NONE) Err=ret;
            
            ret=EXT_I2C_WritePorts_PCA9555(ADR1,0,0);
            if (ret != EXT_I2C_Error_NONE) Err=ret;
            ret=EXT_I2C_WritePorts_PCA9555(ADR2,0,0);
            if (ret != EXT_I2C_Error_NONE) Err=ret;
            ret=EXT_I2C_WritePorts_PCA9555(ADR3,0,0);
            if (ret != EXT_I2C_Error_NONE) Err=ret;
              
    	}		
	}

    for(Scancnt=0x0;Scancnt<0x8;Scancnt++)
    {
        SYS_Detected_PowerDist[Scancnt]=0;
        ret=EXT_I2C_Init_PCA9555_OUT(Scancnt+0x20);
        if (ret==EXT_I2C_Error_NONE) SYS_Detected_PowerDist[Scancnt]=1;
    }     


    return(Err);
    
}
/***********************************************************/
/***********************************************************/
unsigned char PowerDist_GET_I2Cadresses_from_DIB(unsigned char *ADR1,unsigned char *ADR2,unsigned char *ADR3)
{
    // DIB must be already loaded from EEP !
    // returns 0 if DIB entry is valid,
    // returns 1 if DIB entry is invalid
    
    unsigned char t8;       // temp
    unsigned long t32;      // temp
    
    t32=GlobalDIB.RS485adress;
    *ADR3 = (t32 & 0x0F)+0x20;          // LSByte
    t32 = t32 >> 4;
    *ADR2 = (t32 & 0x0F)+0x20;
    t32 = t32 >> 4;
    *ADR1 = (t32 & 0x0F)+0x20;          // MSByte

    t8 = (GlobalDIB.SUBportbit & 0x0F);
    if ((t8) && (t8 < 0xF)) return(0);  // valid
    return(1);                          // invalid
     
}
/***********************************************************/
/***********************************************************/
unsigned char PowerDist_GET_Portbits_from_DIB(unsigned char *PB1,unsigned char *PB2,unsigned char *PB3)
{
    // DIB must be already loaded from EEP !
    // returns 0 if DIB entry is valid,
    // returns 1 if DIB entry is invalid
    
    unsigned char t8;       // temp
     
    t8= GlobalDIB.SUBadress;
    *PB1= t8 & 0x0F;
    t8=t8 >> 4;
    *PB2= t8 & 0x0F;
    
    t8 = (GlobalDIB.SUBportbit & 0xF0);
    t8=t8 >> 4;
    *PB3= t8 & 0x0F;                         
     
    t8 = (GlobalDIB.SUBportbit & 0x0F);
    if ((t8) && (t8 < 0xF)) return(0);  // valid
    return(1);                          // invalid

}
/***********************************************************/
/***********************************************************/
unsigned char PowerDist_MAP_PIBval_2_Portbits(unsigned char *PB1,unsigned char *PB2,unsigned char *PB3)
{
    // returns 0 in PBx if NOT set
    // returns 1 in PBx if SET
    
    // PB3 = MSB, PB1 = LSB
   
    switch(GlobalPIB.Param_EE)
    {
        default:    return(1);
        case 1:     *PB3=0;*PB2=0;*PB1=0;   return(0); break; 
        case 2:     *PB3=0;*PB2=0;*PB1=1;   return(0); break;
        case 3:     *PB3=0;*PB2=1;*PB1=0;   return(0); break;
        case 4:     *PB3=0;*PB2=1;*PB1=1;   return(0); break;
        case 5:     *PB3=1;*PB2=0;*PB1=0;   return(0); break;
        case 6:     *PB3=1;*PB2=0;*PB1=1;   return(0); break;
        case 7:     *PB3=1;*PB2=1;*PB1=0;   return(0); break;
        case 8:     *PB3=1;*PB2=1;*PB1=1;   return(0); break;
    } 
}
/***********************************************************/
/***********************************************************/
unsigned char PowerDist_Test_DIB_invalid(void)
{
    // DIB must be already loaded from EEP !
    // returns 0 if DIB entry is valid,
    // returns 1 if DIB entry is invalid
    
    unsigned char t8;       // temp
     
    t8 = (GlobalDIB.SUBportbit & 0x0F);
    if ((t8) && (t8 < 0xF)) return(0);  // valid
    return(1);                          // invalid

}
