#include "usart_comm.h"
#include "CLI.h"
#include "devices.h"
#include "EEP_adresses.h"
#include "EEP_24FC512.h"

/*************************************************************************/
/* Get DIB for Trackswitch,Signal,Light or ...    */
/*************************************************************************/
// input is global PIB
// data is stored in global DIB
void DEV_4BYTE_GetDIB(unsigned int num)
{
   unsigned char buffer[16];
   unsigned int  adress;
   
   GlobalPIB.DeviceNum=num;
   adress= DEV_DevicetypenumtoEEPAdress();
    
   if (adress==0)               // Error   
   {
    GlobalDIB.EEPAdress=0;
    return;
   }
   EEP_24FC512_ReadMultiByte(adress,&buffer[0],4);
   GlobalDIB.DeviceNum=GlobalPIB.DeviceNum;
   GlobalDIB.RS485adress=buffer[0];
   GlobalDIB.RS485adress=GlobalDIB.RS485adress << 8;
   GlobalDIB.RS485adress+=buffer[1];
   GlobalDIB.SUBadress=buffer[2];
   GlobalDIB.SUBportbit=buffer[3];
   
   
}
/*************************************************************************/
/* Get DIB for locos (short)    */
/*************************************************************************/
void DEV_64BYTE_GetDIB_short(unsigned int num)
{
   unsigned char buffer[16];
   unsigned int  adress;
   
   GlobalPIB.DeviceNum=num;
   adress=DEV_DevicetypenumtoEEPAdress();

   if (adress==0)               // Error   
   {
    GlobalDIB.EEPAdress=0;
    return;
   }
    
   EEP_24FC512_ReadMultiByte(adress,&buffer[0],4);
   GlobalDIB.DeviceNum=GlobalPIB.DeviceNum;
   GlobalDIB.LocoAdress=buffer[0];
   GlobalDIB.LocoAdress=GlobalDIB.LocoAdress << 8;
   GlobalDIB.LocoAdress+=buffer[1];
   GlobalDIB.LocoConfig=buffer[2];


}
/*************************************************************************/
/* Get DIB for locos (long)    */
/*************************************************************************/
void DEV_64BYTE_GetDIB_long(unsigned int num)
{
   unsigned char buffer[96];
   unsigned int  adress;
   unsigned char cnt;

   GlobalPIB.DeviceNum=num;
   adress=DEV_DevicetypenumtoEEPAdress();
   
   if (adress==0)               // Error   
   {
    GlobalDIB.EEPAdress=0;
    return;
   }
   EEP_24FC512_ReadMultiByte(adress,&buffer[0],64);
   GlobalDIB.DeviceNum=GlobalPIB.DeviceNum;
   GlobalDIB.LocoAdress=buffer[0];
   GlobalDIB.LocoAdress=GlobalDIB.LocoAdress << 8;
   GlobalDIB.LocoAdress+=buffer[1];
   GlobalDIB.LocoConfig=buffer[2];
   // use DIB.string
   for (cnt=0;cnt<56;cnt++)
   {
        GlobalDIB.STRING[cnt]=buffer[cnt+8];
        if (GlobalDIB.STRING[cnt] == 0x00) cnt=57;
   }
 
}

/*************************************************************************/
/* Write DIB for Trackswitch,Signal,Light or ...    */
/*************************************************************************/
// input is global PIB
// data is stored in global DIB
void DEV_4BYTE_WriteDIB(unsigned int num)
{
   unsigned char buffer[16];
   unsigned int  adress;
    
   GlobalPIB.DeviceNum=num;
   adress=DEV_DevicetypenumtoEEPAdress();
    
   GlobalDIB.DeviceNum=GlobalPIB.DeviceNum;
   buffer[0]=GlobalDIB.RS485adress >> 8;
   buffer[1]=GlobalDIB.RS485adress & 0xFF;
   buffer[2]=GlobalDIB.SUBadress;
   buffer[3]=GlobalDIB.SUBportbit;
   
   EEP_24FC512_WriteByte(adress+0,buffer[0]);
   EEP_24FC512_WriteByte(adress+1,buffer[1]);
   EEP_24FC512_WriteByte(adress+2,buffer[2]);
   EEP_24FC512_WriteByte(adress+3,buffer[3]);
   
}
/*************************************************************************/
/*                  */
/*************************************************************************/
void DEV_64BYTE_WriteDIB(unsigned int num)
{
   unsigned char buffer[16];
   unsigned int  adress;
   unsigned char cnt;
    
   GlobalPIB.DeviceNum=num;
   adress=DEV_DevicetypenumtoEEPAdress();
   GlobalDIB.DeviceNum=GlobalPIB.DeviceNum;
   buffer[0]=GlobalDIB.LocoAdress >> 8;
   buffer[1]=GlobalDIB.LocoAdress & 0xFF;
   buffer[2]=GlobalDIB.LocoConfig;
   buffer[3]=0x00;
   buffer[4]=0x00;
   buffer[5]=0x00;
   buffer[6]=0x00;
   buffer[7]=0x00;
 
   for (cnt=0;cnt<8;cnt++)
   {
        EEP_24FC512_WriteByte(adress+cnt,buffer[cnt]);
   }  
   for (cnt=8;cnt<64;cnt++)
   {
        EEP_24FC512_WriteByte(adress+cnt,GlobalDIB.STRING[cnt-8]);
   }
}


/*************************************************************************/
/*                  */
/*************************************************************************/
unsigned int DEV_DevicetypenumtoEEPAdress(void)
{
  // use values from gloab PIB
  
  unsigned int RetAdress;
  unsigned int tadr;
  unsigned char CMD;
  
  CMD=DEV_toupper(GlobalPIB.CMD);
  
  switch(CMD)
  {
        default:    
                tadr=0;
                 break;   
        // 4 Byte Blocks
        case CLI_PCMD_S_Trackswitch:
        case CLI_PCMD_S_Decoupler:
        case CLI_PCMD_S_PowerSource:
        case CLI_PCMD_S_Signal:
        case CLI_PCMD_S_Posdetect:
        case CLI_PCMD_S_Light:
        case CLI_PCMD_S_PowerDist:
                 tadr=GlobalPIB.DeviceNum;              // get devicenum
                 tadr = tadr << 2;                      // shift by number per elemnt
                 break;   
  
        case CLI_PCMD_S_Loco:
                 tadr=GlobalPIB.DeviceNum;              // get devicenum
                 tadr = tadr << 6;                      // shift by number per elemnt
                 break;   
            
  }
 
 
  switch(CMD)
  {
    default:     GlobalDIB.EEPAdress=0;                 // store error adress
                 break;
    case CLI_PCMD_S_Trackswitch:
                 RetAdress=EEP_Offset_Trackswitch;      // Set Offset
                 RetAdress+=tadr;                       // add
                 if (RetAdress>EEP_MaxAdress_Trackswitch)
                 {
                    GlobalDIB.EEPAdress=0;               // store error adress
                    RetAdress=0;
                 }
                 else
                 {
                    GlobalDIB.EEPAdress=RetAdress;       // store adress
                 }   
                 break;

    case CLI_PCMD_S_Decoupler:
                 RetAdress=EEP_Offset_Decoupler;      // Set Offset
                 RetAdress+=tadr;                       // add
                 if (RetAdress>EEP_MaxAdress_Decoupler )
                 {
                    GlobalDIB.EEPAdress=0;               // store error adress
                    RetAdress=0;
                 }
                 else
                 {
                    GlobalDIB.EEPAdress=RetAdress;       // store adress
                 }   
                 break;
    case CLI_PCMD_S_PowerSource:
                 RetAdress=0;      // Set Offset
                 GlobalDIB.EEPAdress=0;               // store error adress
                 break;
    case CLI_PCMD_S_Signal:
                 RetAdress=EEP_Offset_Signals;      // Set Offset
                 RetAdress+=tadr;                       // add
                 if (RetAdress>EEP_MaxAdress_Signals )
                 {
                    GlobalDIB.EEPAdress=0;               // store error adress
                    RetAdress=0;
                 }
                 else
                 {
                    GlobalDIB.EEPAdress=RetAdress;       // store adress
                 }   
                 break;
     case CLI_PCMD_S_Posdetect:
                 RetAdress=EEP_Offset_Posdetect  ;      // Set Offset
                 RetAdress+=tadr;                       // add
                 if (RetAdress>EEP_MaxAdress_Posdetect   )
                 {
                    GlobalDIB.EEPAdress=0;               // store error adress
                    RetAdress=0;
                 }
                 else
                 {
                    GlobalDIB.EEPAdress=RetAdress;       // store adress
                 }   
                 break;
      case CLI_PCMD_S_Light:
                 RetAdress=EEP_Offset_Light  ;      // Set Offset
                 RetAdress+=tadr;                       // add
                 if (RetAdress>EEP_MaxAdress_Light )
                 {
                    GlobalDIB.EEPAdress=0;               // store error adress
                    RetAdress=0;
                 }
                 else
                 {
                    GlobalDIB.EEPAdress=RetAdress;       // store adress
                 }   
                 break;
      case CLI_PCMD_S_PowerDist:
                 RetAdress=EEP_Offset_PowerDist  ;      // Set Offset
                 RetAdress+=tadr;                       // add
                 if (RetAdress>EEP_MaxAdress_PowerDist )
                 {
                    GlobalDIB.EEPAdress=0;               // store error adress
                    RetAdress=0;
                 }
                 else
                 {
                    GlobalDIB.EEPAdress=RetAdress;       // store adress
                 }   
                 break;
       case CLI_PCMD_S_Servo:
                 RetAdress=EEP_Offset_Servo  ;      // Set Offset
                 RetAdress+=tadr;                       // add
                 if (RetAdress>EEP_MaxAdress_Servo )
                 {
                    GlobalDIB.EEPAdress=0;               // store error adress
                    RetAdress=0;
                 }
                 else
                 {
                    GlobalDIB.EEPAdress=RetAdress;       // store adress
                 }   
                 break;
       case CLI_PCMD_S_Loco:
                 RetAdress=EEP_Offset_Loco;      // Set Offset
                 RetAdress+=tadr;                       // add
                 if (RetAdress>EEP_MaxAdress_Loco)
                 {
                    GlobalDIB.EEPAdress=0;               // store error adress
                    RetAdress=0;
                 }
                 else
                 {
                    GlobalDIB.EEPAdress=RetAdress;       // store adress
                 }   
                 break;
  }

  return(RetAdress);


}

unsigned char DEV_toupper(unsigned char in)
{
 if ((in>96) && (in <123))
 {
  return(in-32);
 }
 else
 {
  return(in);
 }

}
 


