#include "usart_driver.h"
#include "usart_comm.h"
#include "CLI.h"
#include "SysTimer.h"
/*****************************/
/* FOR AVRGCC 4.33 or higher */
/*****************************/
#include <avr\io.h>




/*******************************************************************/
/*** Get new packet                                                */
/*******************************************************************/
unsigned char UCOMM_GetNewPacket(unsigned char *RxBuffer,unsigned char *TxBuffer,unsigned char RxBufferlen)
{
    unsigned char res;
    unsigned char Packetlen;
    unsigned char cnt_c;
    unsigned char cnt_dev;
    
    
    if (*RxBuffer!=UCOMM_StartPacketChar) return(false);                // first char NOT "#" -> exit
    
    if (RxBufferlen < UCOMM_MinPacketLen) return(false);                // packet length < min -> exit
    
    
    res = UCOMM_ScanUARTBufferForchar(RxBuffer,RxBufferlen,UCOMM_EndPacketChar);
    if (res==false)  return(false);                                      // exit if END-char not found
    
    GlobalMultiPIB.Param_Entrys=0;
    Packetlen=UCOMM_ScanUARTBufferForcharPOS(RxBuffer,RxBufferlen,UCOMM_EndPacketChar);   
    
    
    //**** Find Mode Switch ****//
    if (USART_RS232_MODE!=USART_RS232_MODE_COMMAND)
    {
    //slen=sprintf(&tbuf[0],UCOMM_PacketCMD_ModeCMD);
    //res=UCOMM_ScanUARTBufferForspecial(RxBuffer,RxBufferlen,&tbuf[0],slen);
    res=UCOMM_ScanUARTBufferForMODE(RxBuffer,RxBufferlen);
    if (res==true)
    {
      USART_RS232_MODE=USART_RS232_MODE_COMMAND;          // Set to CMD Mode
      USART_RS232_ClrBuffer();
      LED_Statusreg&=~LED_Status_Flag_Slow;               // CLR LED Flag    
      USART_ODCC_Status|=USART_STATUS_TX_IDLE;            // set flag
      USART_RS232_Status|=USART_STATUS_TX_IDLE;
      *(TxBuffer+0)='#';
      *(TxBuffer+1)='O';
      *(TxBuffer+2)='K';
      *(TxBuffer+3)='%';            
      USART_RS232_StartTx();
      USART_RS232_Waitidle();                              // wait until TX is completely done
      
    }                                                      
    return(false);                                         // exit
    }
    
    //**** Find PING ****//
    res=UCOMM_ScanUARTBufferForPING(RxBuffer,RxBufferlen);
    if (res==true)
    {
      USART_RS232_ClrBuffer();
      LED_Statusreg|=LED_Status_Flag_PING;                // CLR LED Flag    
      USART_ODCC_Status|=USART_STATUS_TX_IDLE;            // set flag
      USART_RS232_Status|=USART_STATUS_TX_IDLE;
      *(TxBuffer+0)='#';
      *(TxBuffer+1)='P';
      *(TxBuffer+2)='O';
      *(TxBuffer+3)='N';
      *(TxBuffer+4)='G';            
      *(TxBuffer+5)='%';            
      USART_RS232_StartTx();
      USART_RS232_Waitidle();                              // wait until TX is completely done
      return(false);                                         // exit
    }


    
    
    // Packet found, get PIB
    GlobalPIB.Adress=UCOMM_HexChar2Value(*(RxBuffer+UCOMM_PackPos_Adress));
    GlobalPIB.Dir=*(RxBuffer+UCOMM_PackPos_Dir);
    GlobalPIB.CMD=*(RxBuffer+UCOMM_PackPos_CMD);       
    GlobalPIB.DeviceNum=UCOMM_TripleHex2INT(*(RxBuffer+UCOMM_PackPos_Device),*(RxBuffer+UCOMM_PackPos_Device+1),*(RxBuffer+UCOMM_PackPos_Device+2));
    
    //-------------------------------//
    //* READ *//               
    //-------------------------------//
    if (GlobalPIB.Dir==UCOMM_DIR_Read)
    {
      GlobalPIB.Checksum=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_Checksum_RW),*(RxBuffer+UCOMM_PackPos_Checksum_RW+1));
      /* Checksum Error */
      if (UCOMM_Checksum(RxBuffer,UCOMM_PackPos_Checksum_RW)!=GlobalPIB.Checksum)
      {
        UCOMM_SendErrorpacket(TxBuffer,UCOMM_Error_Checksum);
        return(false);
      }
      // Packet found, get PIB
      GlobalPIB.Param_EE=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_ParamEE),*(RxBuffer+UCOMM_PackPos_ParamEE+1));
      GlobalPIB.Param_FF=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_ParamFF),*(RxBuffer+UCOMM_PackPos_ParamFF+1));  
    }
    //-------------------------------//
    //* Write *//               
    //-------------------------------//
    if (GlobalPIB.Dir==UCOMM_DIR_Write)
    {
      GlobalPIB.Checksum=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_Checksum_RW),*(RxBuffer+UCOMM_PackPos_Checksum_RW+1));
      /* Checksum Error */
      if (UCOMM_Checksum(RxBuffer,UCOMM_PackPos_Checksum_RW)!=GlobalPIB.Checksum)
      {
        UCOMM_SendErrorpacket(TxBuffer,UCOMM_Error_Checksum);
        return(false);
      }
      // Packet found, get PIB
      GlobalPIB.Param_EE=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_ParamEE),*(RxBuffer+UCOMM_PackPos_ParamEE+1));
      GlobalPIB.Param_FF=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_ParamFF),*(RxBuffer+UCOMM_PackPos_ParamFF+1));  
    }

    //-------------------------------//
    //* Multi Write/Read            *//               
    //-------------------------------//
    if ((GlobalPIB.Dir==UCOMM_DIR_MultiWrite) || (GlobalPIB.Dir==UCOMM_DIR_MultiRead)) 
    {
      if (!Packetlen)
      {
        UCOMM_SendErrorpacket(TxBuffer,UCOMM_Error_Checksum);
        return(false);
      }

      GlobalPIB.Checksum=UCOMM_DualHex2Byte(*(RxBuffer+Packetlen-2),*(RxBuffer+Packetlen-1));
      /* Checksum Error */
      if (UCOMM_Checksum(RxBuffer,Packetlen-2)!=GlobalPIB.Checksum)
      {
        UCOMM_SendErrorpacket(TxBuffer,UCOMM_Error_Checksum);
        return(false);
      }
      
      cnt_dev=0;
      cnt_c=UCOMM_PackPos_Device;
      while((cnt_c+7) < Packetlen)
      {
        cnt_dev++;
        GlobalMultiPIB.DeviceNum[cnt_dev]=UCOMM_TripleHex2INT(*(RxBuffer+cnt_c),*(RxBuffer+cnt_c+1),*(RxBuffer+cnt_c+2));
        cnt_c+=3;
        GlobalMultiPIB.Param_EE[cnt_dev]=UCOMM_DualHex2Byte(*(RxBuffer+cnt_c),*(RxBuffer+cnt_c+1));
        cnt_c+=2;
        GlobalMultiPIB.Param_FF[cnt_dev]=UCOMM_DualHex2Byte(*(RxBuffer+cnt_c),*(RxBuffer+cnt_c+1));
        cnt_c+=2;
      }
      GlobalMultiPIB.Param_Entrys= cnt_dev;
    }
    
    //-------------------------------//
    //* SET / GET / CLEAR			*//               
    //-------------------------------//
    if ((GlobalPIB.Dir==UCOMM_DIR_SET) || (GlobalPIB.Dir==UCOMM_DIR_GET) || (GlobalPIB.Dir==UCOMM_DIR_CLEAR)) 
    {
      GlobalPIB.Checksum=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_Checksum_S),*(RxBuffer+UCOMM_PackPos_Checksum_S+1));
      /* Checksum Error */
      if (UCOMM_Checksum(RxBuffer,UCOMM_PackPos_Checksum_S)!=GlobalPIB.Checksum)
      {
        UCOMM_SendErrorpacket(TxBuffer,UCOMM_Error_Checksum);
        return(false);
      }    
      GlobalPIB.Param_FF=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_ParamFF_S),*(RxBuffer+UCOMM_PackPos_ParamFF_S+1));
      GlobalPIB.Param_EES=UCOMM_TripleHex2INT(*(RxBuffer+UCOMM_PackPos_ParamEE),*(RxBuffer+UCOMM_PackPos_ParamEE+1),*(RxBuffer+UCOMM_PackPos_ParamEE+2));  
      GlobalPIB.Param_GG=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_ParamGG),*(RxBuffer+UCOMM_PackPos_ParamGG+1));  
    
      if ((GlobalPIB.CMD==CLI_PCMD_S_Loco))     	   // if loco -> copy string
      {
	  	 UCOMM_UARTBufferGETstring(RxBuffer,63,UCOMM_PackPos_Checksum_S+2,&GlobalPIB.StringParam[0]);
	  }
	  if ((GlobalPIB.CMD==CLI_PCMD_S_DirectData))     	// if direct -> copy data
      {
	  	 UCOMM_UARTBufferGETxData(RxBuffer,65,UCOMM_PackPos_Checksum_S+2,&GlobalPIB.StringParam[0]);
	  }
    	
	}
    //-------------------------------//
    //* SCAN *//               
    //-------------------------------//
    if (GlobalPIB.Dir==UCOMM_DIR_SCAN)
    {
      GlobalPIB.Checksum=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_Checksum_S),*(RxBuffer+UCOMM_PackPos_Checksum_S+1));
      /* Checksum Error */
      if (UCOMM_Checksum(RxBuffer,UCOMM_PackPos_Checksum_S)!=GlobalPIB.Checksum)
      {
        UCOMM_SendErrorpacket(TxBuffer,UCOMM_Error_Checksum);
        return(false);
      }    
      GlobalPIB.Param_FF=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_ParamFF_S),*(RxBuffer+UCOMM_PackPos_ParamFF_S+1));
      GlobalPIB.Param_EES=UCOMM_TripleHex2INT(*(RxBuffer+UCOMM_PackPos_ParamEE),*(RxBuffer+UCOMM_PackPos_ParamEE+1),*(RxBuffer+UCOMM_PackPos_ParamEE+2));  
      GlobalPIB.Param_GG=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_ParamGG),*(RxBuffer+UCOMM_PackPos_ParamGG+1));  
    
    
    }    
    //-------------------------------//
    //* SEARCH *//               
    //-------------------------------//
    if (GlobalPIB.Dir==UCOMM_DIR_SEARCH)
    {
      GlobalPIB.Checksum=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_Checksum_S),*(RxBuffer+UCOMM_PackPos_Checksum_S+1));
      /* Checksum Error */
      if (UCOMM_Checksum(RxBuffer,UCOMM_PackPos_Checksum_S)!=GlobalPIB.Checksum)
      {
        UCOMM_SendErrorpacket(TxBuffer,UCOMM_Error_Checksum);
        return(false);
      }    
      GlobalPIB.Param_FF=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_ParamFF_S),*(RxBuffer+UCOMM_PackPos_ParamFF_S+1));
      GlobalPIB.Param_EES=UCOMM_TripleHex2INT(*(RxBuffer+UCOMM_PackPos_ParamEE),*(RxBuffer+UCOMM_PackPos_ParamEE+1),*(RxBuffer+UCOMM_PackPos_ParamEE+2));  
      GlobalPIB.Param_GG=UCOMM_DualHex2Byte(*(RxBuffer+UCOMM_PackPos_ParamGG),*(RxBuffer+UCOMM_PackPos_ParamGG+1));  
    
      if ((GlobalPIB.CMD==CLI_PCMD_S_Loco))     	// if loco -> copy string
      {
	  	 UCOMM_UARTBufferGETstring(RxBuffer,63,UCOMM_PackPos_Checksum_S+2,&GlobalPIB.StringParam[0]);
      }
    }      
    
    
    
    
    USART_RS232_ClrBuffer();
    return(true);
}
/*******************************************************************/
/*** Create & send answer packet                                  */
/*******************************************************************/
void UCOMM_SendAnswerpacket(unsigned char *TxBuffer)
{

    unsigned char CHK;
    
    if (GlobalMultiPIB.Param_Entrys)
    {
        UCOMM_SendAnswerpacket_Multi(TxBuffer);
        return;
    }
    
    USART_RS232_Waitidle();
    *TxBuffer=UCOMM_StartPacketChar;
    *(TxBuffer+UCOMM_PackPos_Adress)=UCOMM_Nibble2HexChar(GlobalPIB.Adress);
    *(TxBuffer+UCOMM_PackPos_Dir)=GlobalPIB.Dir;
    *(TxBuffer+UCOMM_PackPos_CMD)= GlobalPIB.CMD;
    *(TxBuffer+UCOMM_PackPos_Device+0)=UCOMM_Nibble2HexChar(((GlobalPIB.DeviceNum)>>8)&0x0F);
    *(TxBuffer+UCOMM_PackPos_Device+1)=UCOMM_Nibble2HexChar(((GlobalPIB.DeviceNum)>>4)&0x0F);
    *(TxBuffer+UCOMM_PackPos_Device+2)=UCOMM_Nibble2HexChar(((GlobalPIB.DeviceNum)>>0)&0x0F);        
    
    // Error marker //
    if (GlobalPIB.Param_EE==UCOMM_Error_marker)       // Error marker ?
    {
      *(TxBuffer+UCOMM_PackPos_ParamEE+0)=UCOMM_Error_marker;
      *(TxBuffer+UCOMM_PackPos_ParamEE+1)=UCOMM_Error_marker;                
    }
    else
    {                                           // no Error marker
      *(TxBuffer+UCOMM_PackPos_ParamEE+0)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_EE)>>4)&0x0F);
      *(TxBuffer+UCOMM_PackPos_ParamEE+1)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_EE)>>0)&0x0F);                
    }
    *(TxBuffer+UCOMM_PackPos_ParamFF+0)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_FF)>>4)&0x0F);
    *(TxBuffer+UCOMM_PackPos_ParamFF+1)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_FF)>>0)&0x0F);  
    CHK=UCOMM_Checksum(TxBuffer,UCOMM_PackPos_Checksum_RW);
    *(TxBuffer+UCOMM_PackPos_Checksum_RW+0)=UCOMM_Nibble2HexChar(((CHK)>>4)&0x0F);
    *(TxBuffer+UCOMM_PackPos_Checksum_RW+1)=UCOMM_Nibble2HexChar(((CHK)>>0)&0x0F);
    *(TxBuffer+UCOMM_PackPos_END_RW)= UCOMM_EndPacketChar;
    USART_RS232_StartTx();
    
    USART_RS232_Waitidle(); // wait until TX is completely done
}
/*******************************************************************/
/*** Create & send answer packet  (Multi)                         */
/*******************************************************************/
void UCOMM_SendAnswerpacket_Multi(unsigned char *TxBuffer)
{

    unsigned char CHK;
    unsigned char cnt_dev;
    unsigned char cnt_c;
    
    
    USART_RS232_Waitidle();
    *TxBuffer=UCOMM_StartPacketChar;
    *(TxBuffer+UCOMM_PackPos_Adress)=UCOMM_Nibble2HexChar(GlobalPIB.Adress);
    *(TxBuffer+UCOMM_PackPos_Dir)=GlobalPIB.Dir;
    *(TxBuffer+UCOMM_PackPos_CMD)= GlobalPIB.CMD;

   // check if num of parameters valid
   if (!GlobalMultiPIB.Param_Entrys) 
   {
     UCOMM_SendErrorpacket(TxBuffer,UCOMM_Error_CMDVAL);
     return;
   }
 
   cnt_c=UCOMM_PackPos_Device;
   for (cnt_dev=1;cnt_dev<(GlobalMultiPIB.Param_Entrys+1);cnt_dev++)
   {              
        
        GlobalPIB.DeviceNum=GlobalMultiPIB.DeviceNum[cnt_dev];
        GlobalPIB.Param_EE=GlobalMultiPIB.Param_EE[cnt_dev];
        GlobalPIB.Param_FF=GlobalMultiPIB.Param_FF[cnt_dev];
        
        *(TxBuffer+cnt_c++)=UCOMM_Nibble2HexChar(((GlobalPIB.DeviceNum)>>8)&0x0F);
        *(TxBuffer+cnt_c++)=UCOMM_Nibble2HexChar(((GlobalPIB.DeviceNum)>>4)&0x0F);
        *(TxBuffer+cnt_c++)=UCOMM_Nibble2HexChar(((GlobalPIB.DeviceNum)>>0)&0x0F);        
    
        // Error marker //
        if (GlobalPIB.Param_EE==UCOMM_Error_marker)       // Error marker ?
        {
         *(TxBuffer+cnt_c++)=UCOMM_Error_marker;
         *(TxBuffer+cnt_c++)=UCOMM_Error_marker;                
        }
        else
        {                                           // no Error marker
         *(TxBuffer+cnt_c++)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_EE)>>4)&0x0F);
         *(TxBuffer+cnt_c++)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_EE)>>0)&0x0F);                
        }
        *(TxBuffer+cnt_c++)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_FF)>>4)&0x0F);
        *(TxBuffer+cnt_c++)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_FF)>>0)&0x0F);  
   }
    
    // Checksum
    CHK=UCOMM_Checksum(TxBuffer,cnt_c);

    *(TxBuffer+cnt_c++)=UCOMM_Nibble2HexChar(((CHK)>>4)&0x0F);
    *(TxBuffer+cnt_c++)=UCOMM_Nibble2HexChar(((CHK)>>0)&0x0F);
    *(TxBuffer+cnt_c++)= UCOMM_EndPacketChar;
    USART_RS232_StartTx();
    
    USART_RS232_Waitidle(); // wait until TX is completely done
}
/*******************************************************************/
/*** Create & send answer packet  (SETUP)                          */
/*******************************************************************/
void UCOMM_SendAnswerpacket_Setup(unsigned char *TxBuffer)
{

    unsigned char CHK;
    unsigned char Pos;
    USART_RS232_Waitidle();
    *TxBuffer=UCOMM_StartPacketChar;
    *(TxBuffer+UCOMM_PackPos_Adress)=UCOMM_Nibble2HexChar(GlobalPIB.Adress);
    *(TxBuffer+UCOMM_PackPos_Dir)=GlobalPIB.Dir;
    *(TxBuffer+UCOMM_PackPos_CMD)= GlobalPIB.CMD;
    *(TxBuffer+UCOMM_PackPos_Device+0)=UCOMM_Nibble2HexChar(((GlobalPIB.DeviceNum)>>8)&0x0F);
    *(TxBuffer+UCOMM_PackPos_Device+1)=UCOMM_Nibble2HexChar(((GlobalPIB.DeviceNum)>>4)&0x0F);
    *(TxBuffer+UCOMM_PackPos_Device+2)=UCOMM_Nibble2HexChar(((GlobalPIB.DeviceNum)>>0)&0x0F);        
    
    // Error marker //
    if (GlobalPIB.Param_EES==UCOMM_Error_marker)       // Error marker ?
    {
      *(TxBuffer+UCOMM_PackPos_ParamEE+0)=UCOMM_Error_marker;
      *(TxBuffer+UCOMM_PackPos_ParamEE+1)=UCOMM_Error_marker;
      *(TxBuffer+UCOMM_PackPos_ParamEE+2)=UCOMM_Error_marker;                       
    }
    else
    {                                           // no Error marker
      *(TxBuffer+UCOMM_PackPos_ParamEE+0)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_EES)>>8)&0x0F);
      *(TxBuffer+UCOMM_PackPos_ParamEE+1)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_EES)>>4)&0x0F);                
      *(TxBuffer+UCOMM_PackPos_ParamEE+2)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_EES)>>0)&0x0F);      
    }
    
    asm("nop;");
    *(TxBuffer+UCOMM_PackPos_ParamFF_S+0)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_FF)>>4)&0x0F);
    *(TxBuffer+UCOMM_PackPos_ParamFF_S+1)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_FF)>>0)&0x0F);
    *(TxBuffer+UCOMM_PackPos_ParamGG+0)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_GG)>>4)&0x0F);
    *(TxBuffer+UCOMM_PackPos_ParamGG+1)=UCOMM_Nibble2HexChar(((GlobalPIB.Param_GG)>>0)&0x0F);

   
    CHK=UCOMM_Checksum(TxBuffer,UCOMM_PackPos_Checksum_S);
    *(TxBuffer+UCOMM_PackPos_Checksum_S+0)=UCOMM_Nibble2HexChar(((CHK)>>4)&0x0F);
    *(TxBuffer+UCOMM_PackPos_Checksum_S+1)=UCOMM_Nibble2HexChar(((CHK)>>0)&0x0F);
    
	  if (((GlobalPIB.CMD==CLI_PCMD_S_Loco) && (GlobalPIB.Dir==UCOMM_DIR_GET )) || (GlobalPIB.CMD==CLI_PCMD_SCAN2))     	// if loco -> copy string
    {
		   Pos=UCOMM_UARTBufferGETstring(&GlobalPIB.StringParam[0],63,0,TxBuffer+UCOMM_PackPos_Checksum_S+2);
	    *(TxBuffer+UCOMM_PackPos_END_SG+Pos)= UCOMM_EndPacketChar;
    }
	  else
	  {
      *(TxBuffer+UCOMM_PackPos_END_SG)= UCOMM_EndPacketChar;
    }
	
					    
    
    USART_RS232_StartTx();
    USART_RS232_Waitidle(); // wait until TX is completely done
}
/*******************************************************************/
/*** Create & send Error  packet                                  */
/*******************************************************************/
void UCOMM_SendErrorpacket(unsigned char *TxBuffer, unsigned char Errnum)
{
     GlobalPIB.Param_EE=UCOMM_Error_marker;
     GlobalPIB.Param_FF=Errnum;
     UCOMM_SendAnswerpacket(TxBuffer);
     USART_RS232_ClrBuffer();
     return;
}     
void UCOMM_SendErrorpacket_Setup(unsigned char *TxBuffer, unsigned char Errnum)
{
     GlobalPIB.Param_EES=UCOMM_Error_marker;
     GlobalPIB.Param_FF=Errnum;
     UCOMM_SendAnswerpacket_Setup(TxBuffer);
     USART_RS232_ClrBuffer();
     return;
}     





/*******************************************************************/
/*** Tools                                                         */
/*******************************************************************/
/*******************************************************************/

/*******************************************************************/

unsigned char UCOMM_UARTBufferGETstring(unsigned char *UART_Buffer,unsigned char BuffertoScanLen,unsigned char BuffertoScanStart, unsigned char *Stringbuffer)
{
	// copies string from start to 0x00 or % into *String. 
	// stops if BuffertoScanlen is done
	
	unsigned char cnt;
	unsigned char retval;
	cnt=BuffertoScanStart;
	while(cnt < BuffertoScanLen)
	{
		*(Stringbuffer++)=*(UART_Buffer+cnt);
		retval=cnt;
    		
		if (*(UART_Buffer+cnt) == 0x00) 
		{
			*(Stringbuffer)=0x00;
		
			cnt=BuffertoScanLen;			
		}
		if (*(UART_Buffer+cnt) == '%') 
		{
			*(Stringbuffer)=0x00;
			cnt=BuffertoScanLen;			
		}
		
		cnt++;
		if (cnt == BuffertoScanLen) 
		{
			*(Stringbuffer)=0x00;
		}
	}
  retval++;
  return(retval);
}
/*******************************************************************/

/*******************************************************************/

unsigned char UCOMM_UARTBufferGETxData(unsigned char *UART_Buffer,unsigned char BuffertoScanLen,unsigned char BuffertoScanStart, unsigned char *Stringbuffer)
{
	// copies string from start to 0x00 or % into *String. 
	// stops if BuffertoScanlen is done
	
	unsigned char cnt;
	unsigned char retval;
	cnt=BuffertoScanStart;
	while(cnt < BuffertoScanLen)
	{
		*(Stringbuffer++)=*(UART_Buffer+cnt);
		retval=cnt;
    		
		if (*(UART_Buffer+cnt) == '%') 
		{
			*(Stringbuffer)=0x00;
			cnt=BuffertoScanLen;			
		}
		
		cnt++;
		if (cnt == BuffertoScanLen) 
		{
			*(Stringbuffer)=0x00;
		}
	}
  retval++;
  return(retval);
}
/*******************************************************************/

/*******************************************************************/
    

unsigned char UCOMM_ScanUARTBufferForMODE(unsigned char *BuffertoScan,unsigned char BuffertoScanLen)
{
    if (BuffertoScanLen<6)       return(false);  // Stringlen < Searchstringlen !
    
    if (*(BuffertoScan+0)!='#') return(false);
    if (*(BuffertoScan+1)!='@') return(false);
    if (*(BuffertoScan+2)!='@') return(false);
    if (*(BuffertoScan+3)!='@') return(false);
    if (*(BuffertoScan+4)!='@') return(false);
    if (*(BuffertoScan+5)!='%') return(false);
    return(true);
}


unsigned char UCOMM_ScanUARTBufferForPING(unsigned char *BuffertoScan,unsigned char BuffertoScanLen)
{
    if (BuffertoScanLen<6)       return(false);  // Stringlen < Searchstringlen !
    
    if (*(BuffertoScan+0)!='#') return(false);
    if (*(BuffertoScan+1)!='P') return(false);
    if (*(BuffertoScan+2)!='I') return(false);
    if (*(BuffertoScan+3)!='N') return(false);
    if (*(BuffertoScan+4)!='G') return(false);
    if (*(BuffertoScan+5)!='%') return(false);
    return(true);
}




unsigned char UCOMM_ScanUARTBufferForchar(unsigned char *BuffertoScan,unsigned char BuffertoScanLen, unsigned char Chartofind)
{
    unsigned char retval;
    unsigned char cnt_outer;
        
    retval=false;

    if (BuffertoScanLen<1)    return(false);                // Stringlen < Searchstringlen !
    
    cnt_outer=0;
    while (cnt_outer < (BuffertoScanLen))                   // run thru buffer
    {
      if (*(BuffertoScan+cnt_outer)==0x00)                  // exit if buffer is end             
        {
            retval=false;
            cnt_outer = (BuffertoScanLen);
        }
      
      
      if (*(BuffertoScan+cnt_outer)==(Chartofind))         // char found
      {
            retval=true;                                   // exit if not char
            cnt_outer = (BuffertoScanLen);
      }  
      cnt_outer++;
    }   
    return(retval);
}
 unsigned char UCOMM_ScanUARTBufferForcharPOS(unsigned char *BuffertoScan,unsigned char BuffertoScanLen, unsigned char Chartofind)
{
    unsigned char retval;
    unsigned char cnt_outer;
        
    retval=0;

    if (BuffertoScanLen<1)    return(0);                // Stringlen < Searchstringlen !
    
    cnt_outer=0;
    while (cnt_outer < (BuffertoScanLen))                   // run thru buffer
    {
      if (*(BuffertoScan+cnt_outer)==0x00)                  // exit if buffer is end             
        {
            retval=0;
            cnt_outer = (BuffertoScanLen);
        }
      
      
      if (*(BuffertoScan+cnt_outer)==(Chartofind))         // char found
      {
            retval=cnt_outer;                              // exit if not char
      }  
      cnt_outer++;
    }   
    return(retval);
}

unsigned char UCOMM_Checksum(unsigned char *Buffer,unsigned char len)
{
 unsigned char CHK;
 unsigned char cnt;
 CHK=0;
  
 for (cnt=0;cnt<len;cnt++)
 {
  CHK+=*(Buffer++);
 } 
 return(CHK);
}


unsigned char UCOMM_HexChar2Value(unsigned char HexChar)
{
    unsigned char res;
    switch(HexChar)
    {
        default:    res=0;break;
        case UCOMM_Error_marker:    res=UCOMM_Error_marker;break;
        case '0':   res=0;break;
        case '1':   res=1;break;
        case '2':   res=2;break;
        case '3':   res=3;break;
        case '4':   res=4;break;
        case '5':   res=5;break;
        case '6':   res=6;break;
        case '7':   res=7;break;
        case '8':   res=8;break;
        case '9':   res=9;break;
        case 'A':   res=10;break;
        case 'B':   res=11;break;
        case 'C':   res=12;break;
        case 'D':   res=13;break;
        case 'E':   res=14;break;
        case 'F':   res=15;break;
    }
    return(res);
}
unsigned char UCOMM_DualHex2Byte(unsigned char HexCharMSB,unsigned char HexCharLSB)
{
       unsigned char Res;
       Res=UCOMM_HexChar2Value(HexCharMSB);
       Res=Res << 4;
       Res+=UCOMM_HexChar2Value(HexCharLSB);
       return(Res);
}
unsigned int UCOMM_QuadHex2INT(unsigned char HexCharMSB,unsigned char HexChar2,unsigned char HexChar1,unsigned char HexChar0)
{
       unsigned int Res;
       Res=UCOMM_HexChar2Value(HexCharMSB);
       Res=Res << 4;
       Res+=UCOMM_HexChar2Value(HexChar2);
       Res=Res << 4;
       Res+=UCOMM_HexChar2Value(HexChar1);
       Res=Res << 4;
       Res+=UCOMM_HexChar2Value(HexChar0);
       return(Res);
}  
unsigned int UCOMM_TripleHex2INT(unsigned char HexCharMSB,unsigned char HexChar1,unsigned char HexChar0)
{
       unsigned int Res;
       Res=UCOMM_HexChar2Value(HexCharMSB);
       Res=Res << 4;
       Res+=UCOMM_HexChar2Value(HexChar1);
       Res=Res << 4;
       Res+=UCOMM_HexChar2Value(HexChar0);
       return(Res);
}


unsigned char  UCOMM_Nibble2HexChar(unsigned char Nibble)
{
    
    unsigned char String;
    switch(Nibble)
    {
        default:    String='x';break;
        case 0:     String='0';break;
        case 1:     String='1';break;
        case 2:     String='2';break;
        case 3:     String='3';break;
        case 4:     String='4';break;
        case 5:     String='5';break;
        case 6:     String='6';break;
        case 7:     String='7';break;
        case 8:     String='8';break;
        case 9:     String='9';break;
        case 10:    String='A';break;
        case 11:    String='B';break;
        case 12:    String='C';break;
        case 13:    String='D';break;
        case 14:    String='E';break;
        case 15:    String='F';break;
        
    }
    return(String);
}

void UCOMM_Value2HexChar(unsigned char Value,unsigned char *String)
{
    
  *(String+0)=UCOMM_Nibble2HexChar(Value >> 8);
  *(String+1)=UCOMM_Nibble2HexChar(Value & 0xFF);
  
}
/*unsigned char UCOMM_ScanUARTBufferForspecial(unsigned char *BuffertoScan,unsigned char BuffertoScanLen, unsigned char *Buffertofind, unsigned char Buffertofindlen)
{
    unsigned char retval;
    unsigned char cnt_outer;
        
    retval=true;

    if (BuffertoScanLen<Buffertofindlen)    return(false);  // Stringlen < Searchstringlen !
    
    cnt_outer=0;
    while (cnt_outer < (Buffertofindlen))                   // run thru buffer
    {
      if (*(BuffertoScan+cnt_outer)==0x00)                  // exit if buffer is end             
        {
            retval=false;
            cnt_outer = (BuffertoScanLen);
        }
      
      
      if (*(BuffertoScan+cnt_outer)!=*(Buffertofind+cnt_outer))       // char not found
      {
            retval=false;                                   // exit if not char
            cnt_outer = (BuffertoScanLen);
      }  
      cnt_outer++;
    }
    return(retval);
}  */
