#ifndef USART_COMM_H
#define USART_COMM_H

#include <avr\io.h>

/****************************************************************/

/* packet control chars */
#define UCOMM_StartPacketChar '#'
#define UCOMM_EndPacketChar   '%'
#define UCOMM_Error_marker    '!'
#define UCOMM_PacketCMD_ModeCMD "#@@@@%"

/* packet settings */
#define UCOMM_MinPacketLen      	6
#define UCOMM_PackPos_Adress    	1
#define UCOMM_PackPos_Dir      	 	2
#define UCOMM_PackPos_CMD       	3
#define UCOMM_PackPos_Device    	4
#define UCOMM_PackPos_ParamEE    	7
#define UCOMM_PackPos_ParamFF    	9
#define UCOMM_PackPos_ParamFF_S  	10
#define UCOMM_PackPos_ParamGG  		12
#define UCOMM_PackPos_Checksum_RW  	11
#define UCOMM_PackPos_Checksum_S   	14
#define UCOMM_PackPos_Checksum_G  	14
#define UCOMM_PackPos_END_RW    	13
#define UCOMM_PackPos_END_SG    	16

/* DIR constants */
#define UCOMM_DIR_Read          'R'
#define UCOMM_DIR_Write         'W'
#define UCOMM_DIR_SET           'S'
#define UCOMM_DIR_GET           'G'
#define UCOMM_DIR_CLEAR         'C'
#define UCOMM_DIR_SCAN          '0'
#define UCOMM_DIR_SEARCH        'E'
#define UCOMM_DIR_MultiRead     'A'
#define UCOMM_DIR_MultiWrite    'M'

/* Error codes */
#define UCOMM_Error_Checksum    	0x96
#define UCOMM_Error_CMDVAL      	0x99
#define UCOMM_Error_None        	0x80
#define UCOMM_Error_DevNumInvalid   0x22


/* flags */
#define PIB_Flag_valid      (1<<0)
#define PIB_Flag_Stringused (1<<2)

/* types */
struct PacketInfoBlock
{
 unsigned char Adress;
 unsigned char Dir;
 unsigned int CMD;
 unsigned int DeviceNum;
 unsigned char Checksum;
 unsigned char Param_EE;  
 unsigned char Param_FF;  
 unsigned int  Param_EES; 
 unsigned char Param_GG;  
 unsigned char StringParam[68];
};
struct MultiPacketInfoBlock
{
 unsigned char Param_Entrys;
 unsigned char Param_EE[17];  
 unsigned char Param_FF[17];  
 unsigned int  DeviceNum[17];
 unsigned char Error[17];
 
  
};




/****************************************************************/
/* variables */
/****************************************************************/
volatile struct PacketInfoBlock GlobalPIB;
volatile struct MultiPacketInfoBlock GlobalMultiPIB;

/****************************************************************/
/* functions */
/****************************************************************/

void UCOMM_SendErrorpacket(unsigned char *TxBuffer,  unsigned char Errnum);
void UCOMM_SendAnswerpacket(unsigned char *TxBuffer );
unsigned char UCOMM_GetNewPacket(unsigned char *RxBuffer,unsigned char *TxBuffer,unsigned char RxBufferlen );
void UCOMM_SendAnswerpacket_Setup(unsigned char *TxBuffer );
void UCOMM_SendAnswerpacket_Multi(unsigned char *TxBuffer);
void UCOMM_SendErrorpacket_Setup(unsigned char *TxBuffer,  unsigned char Errnum);



unsigned char UCOMM_ScanUARTBuffer(unsigned char *BuffertoScan,unsigned char BuffertoScanLen, unsigned char *Buffertofind, unsigned char Buffertofindlen, unsigned char *Pos);
unsigned char UCOMM_HexChar2Value(unsigned char HexChar);
unsigned char UCOMM_DualHex2Byte(unsigned char HexCharMSB,unsigned char HexCharLSB);
unsigned int UCOMM_QuadHex2INT(unsigned char HexCharMSB,unsigned char HexChar2,unsigned char HexChar1,unsigned char HexChar0);
unsigned int UCOMM_TripleHex2INT(unsigned char HexCharMSB,unsigned char HexChar1,unsigned char HexChar0);

unsigned char UCOMM_ScanUARTBufferForMODE(unsigned char *BuffertoScan,unsigned char BuffertoScanLen);
unsigned char UCOMM_ScanUARTBufferForPING(unsigned char *BuffertoScan,unsigned char BuffertoScanLen);

unsigned char UCOMM_UARTBufferGETstring(unsigned char *UART_Buffer,unsigned char BuffertoScanLen,unsigned char BuffertoScanStart, unsigned char *Stringbuffer);
unsigned char UCOMM_UARTBufferGETxData(unsigned char *UART_Buffer,unsigned char BuffertoScanLen,unsigned char BuffertoScanStart, unsigned char *Stringbuffer);
//unsigned char UCOMM_ScanUARTBufferForspecial(unsigned char *BuffertoScan,unsigned char BuffertoScanLen, unsigned char *Buffertofind, unsigned char Buffertofindlen);
unsigned char UCOMM_ScanUARTBufferForchar(unsigned char *BuffertoScan,unsigned char BuffertoScanLen, unsigned char Chartofind);
void UCOMM_Value2HexChar(unsigned char Value,unsigned char *String);
unsigned char UCOMM_Nibble2HexChar(unsigned char Nibble);
unsigned char UCOMM_Checksum(unsigned char *Buffer,unsigned char len);

unsigned char UCOMM_ScanUARTBufferForcharPOS(unsigned char *BuffertoScan,unsigned char BuffertoScanLen, unsigned char Chartofind);
#ifndef false
#define false
#endif

#ifndef true
#define true
#endif












#endif
