#include "usart_driver.h"
#include "usart_comm.h"

#include "SysTimer.h"
#include "Taskman.h"
/*****************************/
/* FOR AVRGCC 4.33 or higher */
/*****************************/
#include <avr\io.h>

//****************************************************************//
//*** RS232                                                    ***//
//****************************************************************//
void USART_RS232_InitINT(void)
{
  USART_RS232_RxBufCntr=0;
  USART_RS232_CTS_INIT;
  USART_RS232_RTS_INIT;
  USART_RS232_CTS_HI;              // NOT ready
  
  USART_RS232_MODE=USART_RS232_MODE_THRU;
  LED_Statusreg|=LED_Status_Flag_Slow;
  
  USART_RS232_Status|= USART_STATUS_TX_IDLE;
  USART_RS232_Status&=~USART_STATUS_TX_LastChr;
  
  UART_CLR232Bufer();
  
  USART_Baudrate_Set(&USART_RS232,USART_RS232_BRHI,USART_RS232_BRLO,USART_RS232_Double);
 
  USART_RS232_TxPininit;
  USART_RS232_RxPininit;

  USART_Format_Set(&USART_RS232, USART_CHSIZE_8BIT_gc, USART_PMODE_DISABLED_gc, false);
  USART_RxdInterruptLevel_Set(&USART_RS232,USART_RS232_INT_LVL);
  // TxINT is disabled until Mode!=THRU
    
}
/***----------------------------------------***/
/*** RX Complete Interrupt Service Routine. ***/
/***----------------------------------------***/
ISR(USART_RS232_RXC_INT) 
{
  unsigned char tdata;
                                                          
  tdata=USART_RS232_DATA;                                 // temp data
  
  #ifdef USART_DEBUG232
  USART_RS232_DATA=tdata;
  #endif
  
  // Check Mode // 
  if (USART_RS232_MODE==USART_RS232_MODE_THRU)           // Mode = THRU ?
  {
      #ifndef USART_DEBUG232
      while(!(USART_ODCC_STATUS & USART_DREIF_bm) );     // wait until last char is gone....
      USART_ODCC_STATUS |= USART_DREIF_bm;
      USART_ODCC_Status&=~USART_STATUS_TX_IDLE;          // clr flag
      USART_ODCC_DATA=tdata;                             // send data away
      #endif  
  }
  // Scan for Packet Start char // 
  if (tdata==UCOMM_StartPacketChar)                      // test for char
  {                                                      // char found   
     USART_RS232_RxBufCntr=0;                            // clr buffer
  }
  
  USART_RS232_RxBuf[USART_RS232_RxBufCntr++]=tdata;
  if (USART_RS232_RxBufCntr==USART_RS232_RxBufferSize) USART_RS232_RxBufCntr = 0;       // reset counter because of > max   
  USART_RS232_RxBuf[USART_RS232_RxBufCntr]=0x00;                                        // next is buffer end until new            
  
  USART_LED_Pulse;                                     // pulse LED a bit...
  
}
/***----------------------------------------***/
/*** TX Complete Interrupt Service Routine. ***/
/***----------------------------------------***/
ISR(USART_RS232_TXC_INT)
{
    
    // buffersize done -> exit
    if (USART_RS232_TxBufCntr>USART_RS232_TxBufferSize)                                  
    {
        USART_RS232_Status|= USART_STATUS_TX_IDLE;
        USART_RS232_Status&=~USART_STATUS_TX_LastChr;
        return;                          
    }    
    
    // last char flag -> exit
    if (USART_RS232_Status & USART_STATUS_TX_LastChr)                                    
    {
        USART_RS232_Status|= USART_STATUS_TX_IDLE;
        USART_RS232_Status&=~USART_STATUS_TX_LastChr;
        return;
    }
    
    USART_RS232_DATA=USART_RS232_TxBuf[USART_RS232_TxBufCntr];                           // send char   
    if (USART_RS232_TxBuf[USART_RS232_TxBufCntr++]==UCOMM_EndPacketChar)                 // END-char found -> exit    
    {
        USART_RS232_Status|= USART_STATUS_TX_LastChr;                                    // set last char flag
    }
    
    USART_LED_Pulse;         

}
/***----------------------------------------***/
/*** TX start                               ***/
/***----------------------------------------***/
void USART_RS232_StartTx(void)
{
    
    unsigned char exit_loop;
    
    Taskman_InitIdleTask(100);
    exit_loop=0;
    
    while(!exit_loop)
    {
        if ((USART_RS232_Status & USART_STATUS_TX_IDLE)) exit_loop=1;
        if (Taskman_IdleTask()==Taskman_Timeout) exit_loop=2;        
    }


    USART_TxdInterruptLevel_Set(&USART_RS232,USART_RS232_TxINT_LVL);

    USART_RS232_Status&=~USART_STATUS_TX_LastChr;                                        // clr last char flag
    USART_RS232_Status&=~USART_STATUS_TX_IDLE;                                         // clr flag
    USART_RS232_TxBufCntr=0;                                                             // clr Buffer Pointer
    USART_RS232_DATA=USART_RS232_TxBuf[USART_RS232_TxBufCntr++];                         // Send first char   
}  

/***----------------------------------------***/
/*** Enable ***/
/***----------------------------------------***/
void USART_RS232_Enable(void)
{
   USART_Rx_Enable(&USART_RS232);
   USART_Tx_Enable(&USART_RS232);
  // USART_RS232_CTS_LO;              // Ready 

}
void USART_RS232_Disable(void)
{
   USART_Rx_Disable(&USART_RS232);
   USART_Tx_Disable(&USART_RS232);
  // USART_RS232_CTS_LO;              // Ready 

}
/***----------------------------------------***/
/*** Packet length ***/
/***----------------------------------------***/
unsigned char USART_RS232_GetPacketLen(void)
{
    return(USART_RS232_RxBufCntr);
}
/***----------------------------------------***/
/*** Clr Buffer ***/
/***----------------------------------------***/
void USART_RS232_ClrBuffer(void)
{
    USART_RS232_RxBufCntr=0;                   // clr buffer
    USART_RS232_RxBuf[USART_RS232_RxBufCntr+0]=0x00;
    USART_RS232_RxBuf[USART_RS232_RxBufCntr+1]=0x00; 
}
/***----------------------------------------***/
/*** Wait until Tx is idle ***/
/***----------------------------------------***/
void USART_RS232_Waitidle(void)
{
    unsigned char exit_loop;
    
    Taskman_InitIdleTask(100);
    exit_loop=0;
    
    while(!exit_loop)
    {
        if ((USART_RS232_Status & USART_STATUS_TX_IDLE)) exit_loop=1;
        if (Taskman_IdleTask()==Taskman_Timeout) exit_loop=2;        
    }


}

//****************************************************************//
//*** OpenDCC                                                  ***//
//****************************************************************//
void USART_ODCC_InitINTRx(void)
{
  USART_ODCC_RxBufCntr=0;                   
  USART_ODCC_CTS_INIT;
  USART_ODCC_RTS_INIT;
  USART_ODCC_CTS_HI;     // NOT ready
  
  
  USART_Format_Set(&USART_ODCC, USART_CHSIZE_8BIT_gc, USART_PMODE_DISABLED_gc, false);
  USART_RxdInterruptLevel_Set(&USART_ODCC,USART_ODCC_INT_LVL);     // set Rx INT ON
  USART_TxdInterruptLevel_Set(&USART_ODCC,USART_TXCINTLVL_OFF_gc); // set Tx INT OFF
  
  USART_Baudrate_Set(&USART_ODCC,USART_RS232_BRHI,USART_RS232_BRLO,USART_RS232_Double);
  
  USART_ODCC_TxPininit;
  USART_ODCC_RxPininit;
  
}

/*** RX Complete Interrupt Service Routine. ***/
ISR(USART_ODCC_RXC_INT) 
{
  unsigned char tdata;
  
  tdata=USART_ODCC_DATA;
  if (USART_RS232_MODE==USART_RS232_MODE_THRU)
  {
      while(!(USART_RS232_STATUS & USART_DREIF_bm) ); // wait until last char is gone....
      USART_RS232_STATUS |= USART_DREIF_bm;
      USART_RS232_DATA=tdata;
      USART_RS232_Status&=~USART_STATUS_TX_IDLE;
  }
  USART_ODCC_RxBuf[USART_ODCC_RxBufCntr++]=tdata;
  if (USART_ODCC_RxBufCntr==USART_ODCC_RxBufferSize) USART_ODCC_RxBufCntr = 0;       // reset counter because of > max   
  USART_LED_Pulse;
}
 
/*** TX Complete Interrupt Service Routine. ***/
ISR(USART_ODCC_TXC_INT)
{
    
    if (USART_ODCC_TxBufCntr>USART_ODCC_TxBufferSize)                                 // buffersize done -> exit 
    {
        USART_ODCC_Status|= USART_STATUS_TX_IDLE;
        USART_ODCC_Status&=~USART_STATUS_TX_LastChr;
        return;                          
    }    
    
    
    if (USART_ODCC_Status & USART_STATUS_TX_LastChr)                                    // last char flag -> exit
    {
        USART_ODCC_Status|= USART_STATUS_TX_IDLE;
        USART_ODCC_Status&=~USART_STATUS_TX_LastChr;
        return;
    }
    
    USART_ODCC_DATA=USART_RS232_TxBuf[USART_ODCC_TxBufCntr];                           // send char   
    if (USART_ODCC_TxBuf[USART_ODCC_TxBufCntr++]==UCOMM_EndPacketChar)                 // END-char found -> exit    
    {
        USART_ODCC_Status|= USART_STATUS_TX_LastChr;                                    // set last char flag
    }
    
    USART_LED_Pulse;        

}

void USART_ODCC_TxByte(unsigned char data)
{
    USART_TxdInterruptLevel_Set(&USART_ODCC,USART_TXCINTLVL_OFF_gc); // set Tx INT OFF
    while(!(USART_ODCC_STATUS & USART_DREIF_bm));                   // wait until buffer is free ....
    USART_ODCC_STATUS |= USART_TXCIF_bm;
    USART_ODCC_DATA=data;
}
void USART_ODCC_WaitTxByteDone(void)
{
    while (!(USART_ODCC_STATUS & USART_TXCIF_bm));
    USART_ODCC_STATUS |= USART_TXCIF_bm;
}




/*** TX start ***/
void USART_ODCC_StartTx(void)
{
//    while(!(USART_ODCC_Status & USART_STATUS_TX_IDLE));                                 // Loop until idle
    unsigned char exit_loop;
    
    Taskman_InitIdleTask(100);
    exit_loop=0;
    
    while(!exit_loop)
    {
        if ((USART_ODCC_Status & USART_STATUS_TX_IDLE)) exit_loop=1;
        if (Taskman_IdleTask()==Taskman_Timeout) exit_loop=2;        
    }

    USART_ODCC_Status&=~USART_STATUS_TX_LastChr;                                        // clr last char flag
    USART_ODCC_Status&=~USART_STATUS_TX_IDLE;                                           // clr flag
    USART_ODCC_TxBufCntr=0;                                                             // clr Buffer Pointer
    USART_ODCC_DATA=USART_ODCC_TxBuf[USART_ODCC_TxBufCntr];                             // Send first char   
}  

/*** Enable ***/
void USART_ODCC_Enable(void)
{
   USART_Rx_Enable(&USART_ODCC);
   USART_Tx_Enable(&USART_ODCC);
 //  USART_ODCC_CTS_LO;              // Ready 
}





/**************************************************************************/
/**** Polled Tx                                                         ***/
/**************************************************************************/
void USART_RS232_TxStringCRLF(unsigned char *String)
{

}

void USART_RS232_TxString(unsigned char *String)
{

}



//****************************************************************//
//*** RS485                                                ***//
//****************************************************************//
void USART_BUS485_Init(void)
{
  USART_RS485_BufCntr=0;                   
  
  USART_RS485_OE_LO; 
  USART_RS485_OE_INIT;
  
  USART_Format_Set(&USART_RS485, USART_CHSIZE_8BIT_gc, USART_PMODE_DISABLED_gc, false);
  USART_RxdInterruptLevel_Set(&USART_RS485,USART_ODCC_INT_LVL);     // set Rx INT ON
    
  USART_Baudrate_Set(&USART_RS485,USART_RS485_BRHI,USART_RS485_BRLO,USART_RS485_Double);
  
  USART_RS485_TxPininit;
  USART_RS485_RxPininit;
  
  USART_BUS485_ModeRx();
  
}

/*** RX Complete Interrupt Service Routine. ***/
ISR(USART_RS485_RXC_INT) 
{
  unsigned char tdata;
  tdata=USART_RS485_DATA;
  USART_RS485_Buf[USART_RS485_BufCntr++]=tdata;
  if (USART_RS485_BufCntr==USART_RS485_BufferSize) USART_RS485_BufCntr = 0;       // reset counter because of > max  
  USART_485_Status|=USART_STATUS_RX_NewChr;            // set NEW DATA Flag
}

void USART_BUS485_ResetRX(void)
{
   USART_RS485_BufCntr=0;
}

/*** Enable ***/
void USART_BUS485_ModeTx(void)
{
   USART_Rx_Disable(&USART_RS485);
   USART_Tx_Enable(&USART_RS485);
   USART_RS485_TxPininit;
   USART_RS485_OE_HI;
   USART_RxdInterruptLevel_Set(&USART_RS485, USART_RXCINTLVL_OFF_gc);
}      

void USART_BUS485_ModeRx(void)
{
   USART_RS485_BufCntr=0;  
   USART_Tx_Disable(&USART_RS485);
   USART_Rx_Enable(&USART_RS485);
   USART_RS485_OE_LO;
   USART_485_Status&=~USART_STATUS_RX_NewChr;  // CLR FLAG
   USART_RxdInterruptLevel_Set(&USART_RS485,USART_RS485_INT_LVL);     // set Rx INT ON
}

void USART_BUS485_TxByte(unsigned char data)
{
    while(!(USART_RS485_STATUS & USART_DREIF_bm));                     // wait until buffer is free ....
    USART_RS485_STATUS |= USART_TXCIF_bm;
    USART_RS485_DATA=data;
}
void USART_BUS485_WaitTxByteDone(void)
{
    while (!(USART_RS485_STATUS & USART_TXCIF_bm));
    USART_RS485_STATUS |= USART_TXCIF_bm;
}

void  USART_BUS485_BREAK(void)
{
      USART_Tx_Disable(&USART_RS485);
      USART_RS485_TxPinLO;
      USART_485_Timeout=2;
      while(USART_485_Timeout);      
      USART_RS485_TxPinHI;
      USART_Tx_Enable(&USART_RS485);
}
/**************************************************************************/
/**** Baudrate                                                         ***/
/**************************************************************************/

void USART_Baudrate_Set(USART_t *UART, unsigned char BRvalueHI, unsigned char BRvalueLO,unsigned char Speedmode)
{
    (UART)-> BAUDCTRLB = BRvalueHI & 0x0F; // BSCALE =0
    (UART)-> BAUDCTRLA = BRvalueLO;
    if (Speedmode)
    {
        (UART)-> CTRLB|=(1<<USART_CLK2X_bp);     // ON
    }
    else
    {
        (UART)-> CTRLB&=~(1<<USART_CLK2X_bp);    // OFF
    }    
}	


//*************************************************************//
//*** UART Timer init                                       ***//
//*************************************************************//
void UARTTimer_Init(void)
{
  TCD1_INTFLAGS|=(1<<0);              // clr OvFl
  TCD1_CNT=0xFFE6;                    // Reload = 20 -> 
  TCD1_CTRLA|=(1<<0)+(1<<2);          // MCK/64 = 500khz
  TCD1_INTCTRLA=RTC_OVFINTLVL_LO_gc;  // Set LoLevelINT


  
  // Timer is init to ~ 50s
}

//*************************************************************//
//*** 50s Timer Overflow INT                              ***//
//*************************************************************//
ISR(TCD1_OVF_vect)
{
    TCD1_CNT=0xFFE6;
    
    if (USART_485_Timeout)  USART_485_Timeout--;
    if (USART_ODCC_Timeout) USART_ODCC_Timeout--;
    
    // Handle RTS/CTS in THRU mode
    if (USART_RS232_MODE==USART_RS232_MODE_THRU)
    {
      // Handle RTS/CTS THRU
      if (USART_ODCC_RTS_IN)   
      {
        USART_RS232_CTS_HI;              // NOT ready
      }
      else
      {
        USART_RS232_CTS_LO;              // ready
      }
     }
    

     

}   


//*************************************************************//
//*** Clear complete RS232 Buffer, fill with 0x00           ***//
//*************************************************************//
void UART_CLR232Bufer(void)
{
  for (USART_RS232_RxBufCntr=0;USART_RS232_RxBufCntr<USART_RS232_RxBufferSize;USART_RS232_RxBufCntr++)
  {USART_RS232_RxBuf[USART_RS232_RxBufCntr]=0;}
  
  USART_RS232_RxBufCntr=0;

}
