/***********************************************************************/

#ifndef USART_DRIVER_H
#define USART_DRIVER_H
#include <avr\io.h>

/* Universal Synchronous/Asynchronous Receiver/Transmitter */


/*** ATTENTION ***/
/* 
 RS232-CTS is an OUTPUT Signal   LO=ready
 RS232-RTS is an INPUT Signal    LO=ready
 
 ODCC-CTS is an OUTPUT Signal    LO=ready
 ODCC-RTS is an INPUT Signal     LO=ready
 
 ---> Rx/tx and CTS/RTS must be connected crossover !!!
 
 
 *** CTS ***
  
  CTS = HI (LO on RS232) to STOP transfer
  CTS = LO (HI on RS232) to START transfer
  
*/ 

//#define USART_DEBUG232  1

#define USART_DEFAULT_38k4 1

/*****************************/
/* FOR AVRGCC 4.33 or higher */
/*****************************/

/*********************************/
/**** defines Interface #RS232 ***/
/*********************************/
#define USART_RS232_DATA    USARTD1_DATA  
#define USART_RS232_STATUS  USARTD1_STATUS 
#define USART_RS232_CTRLA   USARTD1_CTRLA  
#define USART_RS232_CTRLB   USARTD1_CTRLB  
#define USART_RS232_CTRLC   USARTD1_CTRLC  
#define USART_RS232_BAUDA   USARTD1_BAUDCTRLA  
#define USART_RS232_BAUDB   USARTD1_BAUDCTRLB
  
#define USART_RS232_RXC_INT USARTD1_RXC_vect
#define USART_RS232_TXC_INT USARTD1_TXC_vect
#define USART_RS232_INT_LVL USART_RXCINTLVL_LO_gc
#define USART_RS232_TxINT_LVL USART_TXCINTLVL_LO_gc

#define USART_RS232_TxPininit PORTD.DIRSET = PIN7_bm   
#define USART_RS232_RxPininit PORTD.DIRCLR = PIN6_bm

#define USART_RS232         USARTD1

#define  USART_RS232_RxBufferSize   132
#define  USART_RS232_TxBufferSize   132
volatile unsigned char USART_RS232_RxBuf[USART_RS232_RxBufferSize];
volatile unsigned char USART_RS232_RxBufCntr;
volatile unsigned char USART_RS232_TxBuf[USART_RS232_TxBufferSize];
volatile unsigned char USART_RS232_TxBufCntr;
volatile unsigned char USART_RS232_Status;


// special Pins
#define USART_RS232_CTS_INIT    PORTD.DIRSET = PIN5_bm
#define USART_RS232_RTS_INIT    PORTD.DIRCLR = PIN4_bm
#define USART_RS232_CTS_HI      PORTD.OUTSET = PIN5_bm
#define USART_RS232_CTS_LO      PORTD.OUTCLR = PIN5_bm
#define USART_RS232_RTS_IN      (PORTD.IN&PIN4_bm)

// Baudrate
#ifndef USART_DEFAULT_38k4
#define USART_RS232_BSEL        UART_BAUD_19200_BSEL
#define USART_RS232_BSCALE      UART_BAUD_19200_BSCALE
#define USART_RS232_Double      UART_BAUD_19200_Double
#define USART_RS232_BRHI        0        
#define USART_RS232_BRLO        UART_BAUD_19200_BSEL
#endif
#ifdef USART_DEFAULT_38k4
#define USART_RS232_BSEL        UART_BAUD_38400_BSEL
#define USART_RS232_BSCALE      UART_BAUD_38400_BSCALE
#define USART_RS232_Double      UART_BAUD_38400_Double
#define USART_RS232_BRHI        0        
#define USART_RS232_BRLO        UART_BAUD_38400_BSEL
#endif

/*********************************/
/**** defines Interface #ODCC ****/
/*********************************/
#define USART_ODCC_DATA    USARTD0_DATA  
#define USART_ODCC_STATUS  USARTD0_STATUS 
#define USART_ODCC_CTRLA   USARTD0_CTRLA  
#define USART_ODCC_CTRLB   USARTD0_CTRLB  
#define USART_ODCC_CTRLC   USARTD0_CTRLC  
#define USART_ODCC_BAUDA   USARTD0_BAUDCTRLA  
#define USART_ODCC_BAUDB   USARTD0_BAUDCTRLB
  
#define USART_ODCC_TxPininit PORTD.DIRSET = PIN3_bm   
#define USART_ODCC_RxPininit PORTD.DIRCLR = PIN2_bm

#define USART_ODCC_RXC_INT USARTD0_RXC_vect
#define USART_ODCC         USARTD0

#define USART_ODCC_RXC_INT USARTD0_RXC_vect
#define USART_ODCC_TXC_INT USARTD0_TXC_vect
#define USART_ODCC_INT_LVL USART_RXCINTLVL_LO_gc


#define   USART_ODCC_RxBufferSize   68
#define   USART_ODCC_TxBufferSize   68
volatile unsigned char  USART_ODCC_RxBuf[USART_ODCC_RxBufferSize];
volatile unsigned char  USART_ODCC_RxBufCntr;
volatile unsigned char  USART_ODCC_TxBuf[USART_ODCC_TxBufferSize];
volatile unsigned char  USART_ODCC_TxBufCntr;
volatile unsigned char  USART_ODCC_Status;
volatile unsigned int   USART_ODCC_Timeout;

// special Pins

#define USART_ODCC_CTS_INIT    PORTD.DIRSET = PIN1_bm
#define USART_ODCC_RTS_INIT    PORTD.DIRCLR = PIN0_bm
#define USART_ODCC_CTS_HI      PORTD.OUTSET = PIN1_bm
#define USART_ODCC_CTS_LO      PORTD.OUTCLR = PIN1_bm

#define USART_ODCC_RTS_IN      (PORTD.IN&PIN0_bm)


 // Baudrate
#ifndef USART_DEFAULT_38k4 
#define USART_ODCC_BSEL        UART_BAUD_19200_BSEL
#define USART_ODCC_BSCALE      UART_BAUD_19200_BSCALE
#define USART_ODCC_Double      UART_BAUD_19200_Double
#endif
#ifdef USART_DEFAULT_38k4
#define USART_ODCC_BSEL        UART_BAUD_38400_BSEL
#define USART_ODCC_BSCALE      UART_BAUD_38400_BSCALE
#define USART_ODCC_Double      UART_BAUD_38400_Double
#endif

/*********************************/
/**** defines Interface #RS485 ***/
/*********************************/
#define USART_RS485_DATA    USARTC0_DATA  
#define USART_RS485_STATUS  USARTC0_STATUS 
#define USART_RS485_CTRLA   USARTC0_CTRLA  
#define USART_RS485_CTRLB   USARTC0_CTRLB  
#define USART_RS485_CTRLC   USARTC0_CTRLC  
#define USART_RS485_BAUDA   USARTC0_BAUDCTRLA  
#define USART_RS485_BAUDB   USARTC0_BAUDCTRLB
  
#define USART_RS485_RXC_INT USARTC0_RXC_vect
#define USART_RS485_INT_LVL USART_RXCINTLVL_LO_gc

#define USART_RS485         USARTC0

#define USART_RS485_TxPininit PORTC.DIRSET = PIN3_bm   
#define USART_RS485_RxPininit PORTC.DIRCLR = PIN2_bm

#define USART_RS485_TxPinHI   PORTC.OUTSET = PIN3_bm
#define USART_RS485_TxPinLO   PORTC.OUTCLR = PIN3_bm


#define USART_RS485_BufferSize   68
volatile unsigned char USART_RS485_Buf[USART_RS485_BufferSize];
volatile unsigned char USART_RS485_BufCntr;


// special Pins
#define USART_RS485_OE_INIT    PORTC.DIRSET = PIN4_bm
#define USART_RS485_OE_HI      PORTC.OUTSET = PIN4_bm
#define USART_RS485_OE_LO      PORTC.OUTCLR = PIN4_bm

 // Baudrate
#define USART_RS485_BSEL        UART_BAUD_250k_BSEL
#define USART_RS485_BSCALE      UART_BAUD_250k_BSCALE
#define USART_RS485_BRHI        0        
#define USART_RS485_BRLO        UART_BAUD_250k_BSEL
#define USART_RS485_Double      UART_BAUD_250k_Double



//-------------------------------------------------------------------------//

#define  USART_RS232_MODE_THRU     0
#define  USART_RS232_MODE_COMMAND  1
volatile unsigned char USART_RS232_MODE;


volatile unsigned int USART_485_Timeout;
volatile unsigned int USART_485_Status;

#define USART_STATUS_TX_IDLE        (1<<0)
#define USART_STATUS_TX_LastChr     (1<<1)
#define USART_STATUS_RX_NewChr      (1<<2)




//-------------------------------------------------------------------------//




/*****************************************************************************/
// predefined values for 32 MHz !
#define UART_BAUD_19200_BSEL    103
#define UART_BAUD_19200_Double  0
#define UART_BAUD_19200_BSCALE  0

#define UART_BAUD_38400_BSEL    51
#define UART_BAUD_38400_Double  0
#define UART_BAUD_38400_BSCALE  0

#define UART_BAUD_115200_BSEL    34  
#define UART_BAUD_115200_Double  1
#define UART_BAUD_115200_BSCALE  0


#define UART_BAUD_250k_BSEL    7
#define UART_BAUD_250k_Double  0
#define UART_BAUD_250k_BSCALE  0


//-------------------------------------------------------------------------//

void USART_Baudrate_Set(USART_t *UART, unsigned char BRvalueHI, unsigned char BRvalueLO,unsigned char Speedmode);
void UARTTimer_Init(void);

void USART_RS232_InitINT(void);
void USART_RS232_StartTx(void);
void USART_RS232_Enable(void);
unsigned char USART_RS232_GetPacketLen(void);
void USART_RS232_ClrBuffer(void);
void USART_RS232_Waitidle(void);
void USART_RS232_Disable(void);
void UART_CLR232Bufer(void);


void USART_ODCC_InitINTRx(void);
void USART_ODCC_StartTx(void);
void USART_ODCC_Enable(void);
void USART_ODCC_TxByte(unsigned char data);
void USART_ODCC_WaitTxByteDone(void);




void USART_BUS485_Init(void);
void USART_BUS485_TxByte(unsigned char data);
void USART_BUS485_WaitTxByteDone(void);
void USART_BUS485_ModeTx(void);
void USART_BUS485_ModeRx(void);
void USART_BUS485_BREAK(void);
void USART_BUS485_ResetRX(void);

/*****************************************************************************/
// below stuff from XMEGA header file and tool functions 

//Baudrate: BSEL = (MCLK/(16*Baud)) -1  // BAUD=MCLK/(16*(BSEL+1))      [SingleSpeed, BSCALE=0]
//Baudrate: BSEL = (1/(2^BSCALE)) * (MCLK/(16*Baud)) -1)  // BAUD=MCLK/(16*((2^BSCALE*BSEL)+1))      [SingleSpeed, BSCALE<0]
// 19200 @ 32M = 103 / SingleSpeed, BSCALE=0 (-> Error = ~ 0.15 % )
// 115k2 @ 32M = 16,36 (???)
// 250k  @ 32M = 7  / SingleSpeed, BSCALE=0

/**** possible Interfaces ***/
/*
  USARTC0
  USARTC1
  USARTD0
  USARTD1
  USARTE0
  USARTE1
  USARTF0
  USARTF1
*/

/* USART interrupt vectors */
/*#define USARTC0_RXC_vect_num  25
#define USARTC0_RXC_vect      _VECTOR(25)  /  Reception Complete Interrupt  /
#define USARTC0_DRE_vect_num  26
#define USARTC0_DRE_vect      _VECTOR(26)  /  Data Register Empty Interrupt  /
#define USARTC0_TXC_vect_num  27
#define USARTC0_TXC_vect      _VECTOR(27)  /  Transmission Complete Interrupt  /

/  USARTC1 interrupt vectors  /
#define USARTC1_RXC_vect_num  28
#define USARTC1_RXC_vect      _VECTOR(28)  /  Reception Complete Interrupt  /
#define USARTC1_DRE_vect_num  29
#define USARTC1_DRE_vect      _VECTOR(29)  /  Data Register Empty Interrupt  /
#define USARTC1_TXC_vect_num  30
#define USARTC1_TXC_vect      _VECTOR(30)  /  Transmission Complete Interrupt  /

*/



#include "avr_compiler.h"


/* Macros. */

/*! \brief Macro that sets the USART frame format.
 *
 *  Sets the frame format, Frame Size, parity mode and number of stop bits.
 *
 *  \param _usart        Pointer to the USART module
 *  \param _charSize     The character size. Use USART_CHSIZE_t type.
 *  \param _parityMode   The parity Mode. Use USART_PMODE_t type.
 *  \param _twoStopBits  Enable two stop bit mode. Use bool type.
 */
#define USART_Format_Set(_usart, _charSize, _parityMode, _twoStopBits)         \
	(_usart)->CTRLC = (uint8_t) _charSize | _parityMode |                      \
	                  (_twoStopBits ? USART_SBMODE_bm : 0)


/*! \brief Set USART baud rate.
 *
 *  Sets the USART's baud rate register.
 *
 *  UBRR_Value   : Value written to UBRR
 *  ScaleFactor  : Time Base Generator Scale Factor
 *
 *  Equation for calculation of BSEL value in asynchronous normal speed mode:
 *  	If ScaleFactor >= 0
 *  		BSEL = ((I/O clock frequency)/(2^(ScaleFactor)*16*Baudrate))-1
 *  	If ScaleFactor < 0
 *  		BSEL = (1/(2^(ScaleFactor)*16))*(((I/O clock frequency)/Baudrate)-1)
 *
 *	\note See XMEGA manual for equations for calculation of BSEL value in other
 *        modes.
 *
 *  \param _usart          Pointer to the USART module.
 *  \param _bselValue      Value to write to BSEL part of Baud control register.
 *                         Use uint16_t type.
 *  \param _bScaleFactor   USART baud rate scale factor.
 *                         Use uint8_t type
 */
 /*
#define USART_Baudrate_Set(_usart, _bselValue, _bScaleFactor)                  \
	(_usart)->BAUDCTRLA =(uint8_t)_bselValue;                                           \
	(_usart)->BAUDCTRLB =(_bScaleFactor << USART_BSCALE0_bp)|(_bselValue >> 8)
   */

/*! \brief Enable USART receiver.
 *
 *  \param _usart    Pointer to the USART module
 */
#define USART_Rx_Enable(_usart) ((_usart)->CTRLB |= USART_RXEN_bm)


/*! \brief Disable USART receiver.
 *
 *  \param _usart Pointer to the USART module.
 */
#define USART_Rx_Disable(_usart) ((_usart)->CTRLB &= ~USART_RXEN_bm)


/*! \brief Enable USART transmitter.
 *
 *  \param _usart Pointer to the USART module.
 */
#define USART_Tx_Enable(_usart)	((_usart)->CTRLB |= USART_TXEN_bm)


/*! \brief Disable USART transmitter.
 *
 *  \param _usart Pointer to the USART module.
 */
#define USART_Tx_Disable(_usart) ((_usart)->CTRLB &= ~USART_TXEN_bm)


/*! \brief Set USART RXD interrupt level.
 *
 *  Sets the interrupt level on RX Complete interrupt.
 *
 *  \param _usart        Pointer to the USART module.
 *  \param _rxdIntLevel  Interrupt level of the RXD interrupt.
 *                       Use USART_RXCINTLVL_t type.
 */
#define USART_RxdInterruptLevel_Set(_usart, _rxdIntLevel)                      \
	((_usart)->CTRLA = ((_usart)->CTRLA & ~USART_RXCINTLVL_gm) | _rxdIntLevel)


/*! \brief Set USART TXD interrupt level.
 *
 *  Sets the interrupt level on TX Complete interrupt.
 *
 *  \param _usart        Pointer to the USART module.
 *  \param _txdIntLevel  Interrupt level of the TXD interrupt.
 *                       Use USART_TXCINTLVL_t type.
 */
#define USART_TxdInterruptLevel_Set(_usart, _txdIntLevel)                      \
	(_usart)->CTRLA = ((_usart)->CTRLA & ~USART_TXCINTLVL_gm) | _txdIntLevel



/*! \brief Set USART DRE interrupt level.
 *
 *  Sets the interrupt level on Data Register interrupt.
 *
 *  \param _usart        Pointer to the USART module.
 *  \param _dreIntLevel  Interrupt level of the DRE interrupt.
 *                       Use USART_DREINTLVL_t type.
 */
#define USART_DreInterruptLevel_Set(_usart, _dreIntLevel)                      \
	(_usart)->CTRLA = ((_usart)->CTRLA & ~USART_DREINTLVL_gm) | _dreIntLevel


/*! \brief Set the mode the USART run in.
 *
 * Set the mode the USART run in. The default mode is asynchronous mode.
 *
 *  \param  _usart       Pointer to the USART module register section.
 *  \param  _usartMode   Selects the USART mode. Use  USART_CMODE_t type.
 *
 *  USART modes:
 *  - 0x0        : Asynchronous mode.
 *  - 0x1        : Synchronous mode.
 *  - 0x2        : IrDA mode.
 *  - 0x3        : Master SPI mode.
 */
#define USART_SetMode(_usart, _usartMode)                                      \
	((_usart)->CTRLC = ((_usart)->CTRLC & (~USART_CMODE_gm)) | _usartMode)



/*! \brief Check if data register empty flag is set.
 *
 *  \param _usart      The USART module.
 */
#define USART_IsTXDataRegisterEmpty(_usart) (((_usart)->STATUS & USART_DREIF_bm) != 0)



/*! \brief Put data (5-8 bit character).
 *
 *  Use the macro USART_IsTXDataRegisterEmpty before using this function to
 *  put data to the TX register.
 *
 *  \param _usart      The USART module.
 *  \param _data       The data to send.
 */
#define USART_PutChar(_usart, _data) ((_usart)->DATA = _data)



/*! \brief Checks if the RX complete interrupt flag is set.
 *
 *   Checks if the RX complete interrupt flag is set.
 *
 *  \param _usart     The USART module.
 */
#define USART_IsRXComplete(_usart) (((_usart)->STATUS & USART_RXCIF_bm) != 0)




/*! \brief Get received data (5-8 bit character).
 *
 *  This macro reads out the RX register.
 *  Use the macro USART_RX_Complete to check if anything is received.
 *
 *  \param _usart     The USART module.
 *
 *  \retval           Received data.
 */
#define USART_GetChar(_usart)  ((_usart)->DATA)

//*** defines from iox128a1.h ***//

/*

#define USART_RXCIF_bm  0x80  /  Receive Interrupt Flag bit mask.  /
#define USART_RXCIF_bp  7  /  Receive Interrupt Flag bit position.  /

#define USART_TXCIF_bm  0x40  /  Transmit Interrupt Flag bit mask.  /
#define USART_TXCIF_bp  6  /  Transmit Interrupt Flag bit position.  /

#define USART_DREIF_bm  0x20  /  Data Register Empty Flag bit mask.  /
#define USART_DREIF_bp  5  /  Data Register Empty Flag bit position.  /

#define USART_FERR_bm  0x10  /  Frame Error bit mask.  /
#define USART_FERR_bp  4  /  Frame Error bit position.  /

#define USART_BUFOVF_bm  0x08  /  Buffer Overflow bit mask.  /
#define USART_BUFOVF_bp  3  /  Buffer Overflow bit position.  /

#define USART_PERR_bm  0x04  /  Parity Error bit mask.  /
#define USART_PERR_bp  2  /  Parity Error bit position.  /

#define USART_RXB8_bm  0x01  /  Receive Bit 8 bit mask.  /
#define USART_RXB8_bp  0  /  Receive Bit 8 bit position.  /


/  USART.CTRLA  bit masks and bit positions  /
#define USART_RXCINTLVL_gm  0x30  /  Receive Interrupt Level group mask.  /
#define USART_RXCINTLVL_gp  4  /  Receive Interrupt Level group position.  /
#define USART_RXCINTLVL0_bm  (1<<4)  /  Receive Interrupt Level bit 0 mask.  /
#define USART_RXCINTLVL0_bp  4  /  Receive Interrupt Level bit 0 position.  /
#define USART_RXCINTLVL1_bm  (1<<5)  /  Receive Interrupt Level bit 1 mask.  /
#define USART_RXCINTLVL1_bp  5  /  Receive Interrupt Level bit 1 position.  /

#define USART_TXCINTLVL_gm  0x0C  /  Transmit Interrupt Level group mask.  /
#define USART_TXCINTLVL_gp  2  /  Transmit Interrupt Level group position.  /
#define USART_TXCINTLVL0_bm  (1<<2)  /  Transmit Interrupt Level bit 0 mask.  /
#define USART_TXCINTLVL0_bp  2  /  Transmit Interrupt Level bit 0 position.  /
#define USART_TXCINTLVL1_bm  (1<<3)  /  Transmit Interrupt Level bit 1 mask.  /
#define USART_TXCINTLVL1_bp  3  /  Transmit Interrupt Level bit 1 position.  /

#define USART_DREINTLVL_gm  0x03  /  Data Register Empty Interrupt Level group mask.  /
#define USART_DREINTLVL_gp  0  /  Data Register Empty Interrupt Level group position.  /
#define USART_DREINTLVL0_bm  (1<<0)  /  Data Register Empty Interrupt Level bit 0 mask.  /
#define USART_DREINTLVL0_bp  0  /  Data Register Empty Interrupt Level bit 0 position.  /
#define USART_DREINTLVL1_bm  (1<<1)  /  Data Register Empty Interrupt Level bit 1 mask.  /
#define USART_DREINTLVL1_bp  1  /  Data Register Empty Interrupt Level bit 1 position.  /


/  USART.CTRLB  bit masks and bit positions  /
#define USART_RXEN_bm  0x10  /  Receiver Enable bit mask.  /
#define USART_RXEN_bp  4  /  Receiver Enable bit position.  /

#define USART_TXEN_bm  0x08  /  Transmitter Enable bit mask.  /
#define USART_TXEN_bp  3  /  Transmitter Enable bit position.  /

#define USART_CLK2X_bm  0x04  /  Double transmission speed bit mask.  /
#define USART_CLK2X_bp  2  /  Double transmission speed bit position.  /

#define USART_MPCM_bm  0x02  /  Multi-processor Communication Mode bit mask.  /
#define USART_MPCM_bp  1  /  Multi-processor Communication Mode bit position.  /

#define USART_TXB8_bm  0x01  /  Transmit bit 8 bit mask.  /
#define USART_TXB8_bp  0  /  Transmit bit 8 bit position.  /


/  USART.CTRLC  bit masks and bit positions  /
#define USART_CMODE_gm  0xC0  /  Communication Mode group mask.  /
#define USART_CMODE_gp  6  /  Communication Mode group position.  /
#define USART_CMODE0_bm  (1<<6)  /  Communication Mode bit 0 mask.  /
#define USART_CMODE0_bp  6  /  Communication Mode bit 0 position.  /
#define USART_CMODE1_bm  (1<<7)  /  Communication Mode bit 1 mask.  /
#define USART_CMODE1_bp  7  /  Communication Mode bit 1 position.  /

#define USART_PMODE_gm  0x30  /  Parity Mode group mask.  /
#define USART_PMODE_gp  4  /  Parity Mode group position.  /
#define USART_PMODE0_bm  (1<<4)  /  Parity Mode bit 0 mask.  /
#define USART_PMODE0_bp  4  /  Parity Mode bit 0 position.  /
#define USART_PMODE1_bm  (1<<5)  /  Parity Mode bit 1 mask.  /
#define USART_PMODE1_bp  5  /  Parity Mode bit 1 position.  /

#define USART_SBMODE_bm  0x08  /  Stop Bit Mode bit mask.  /
#define USART_SBMODE_bp  3  /  Stop Bit Mode bit position.  /

#define USART_CHSIZE_gm  0x07  /  Character Size group mask.  /
#define USART_CHSIZE_gp  0  /  Character Size group position.  /
#define USART_CHSIZE0_bm  (1<<0)  /  Character Size bit 0 mask.  /
#define USART_CHSIZE0_bp  0  /  Character Size bit 0 position.  /
#define USART_CHSIZE1_bm  (1<<1)  /  Character Size bit 1 mask.  /
#define USART_CHSIZE1_bp  1  /  Character Size bit 1 position.  /
#define USART_CHSIZE2_bm  (1<<2)  /  Character Size bit 2 mask.  /
#define USART_CHSIZE2_bp  2  /  Character Size bit 2 position.  /
*/

#endif
